# YORE: Bump 2: Remove file.

from __future__ import annotations

from typing import Literal

from failprint import lazy


@lazy(name="pytest")
def run(
    *paths: str,
    config_file: str | None = None,
    select: str | None = None,
    select_markers: str | None = None,
    markers: bool | None = None,
    exitfirst: bool | None = None,
    fixtures: bool | None = None,
    fixtures_per_test: bool | None = None,
    pdb: bool | None = None,
    pdbcls: str | None = None,
    trace: bool | None = None,
    capture: str | None = None,
    runxfail: bool | None = None,
    last_failed: bool | None = None,
    failed_first: bool | None = None,
    new_first: bool | None = None,
    cache_show: str | None = None,
    cache_clear: bool | None = None,
    last_failed_no_failures: Literal["all", "none"] | None = None,
    stepwise: bool | None = None,
    stepwise_skip: bool | None = None,
    durations: int | None = None,
    durations_min: int | None = None,
    verbose: bool | None = None,
    no_header: bool | None = None,
    no_summary: bool | None = None,
    quiet: bool | None = None,
    verbosity: int | None = None,
    show_extra_summary: str | None = None,
    disable_pytest_warnings: bool | None = None,
    showlocals: bool | None = None,
    no_showlocals: bool | None = None,
    traceback: Literal["auto", "long", "short", "line", "native", "no"] | None = None,
    show_capture: Literal["no", "stdout", "stderr", "log", "all"] | None = None,
    full_trace: bool | None = None,
    color: str | None = None,
    code_highlight: bool | None = None,
    pastebin: str | None = None,
    junit_xml: str | None = None,
    junit_prefix: str | None = None,
    pythonwarnings: str | None = None,
    maxfail: int | None = None,
    strict_config: bool | None = None,
    strict_markers: bool | None = None,
    continue_on_collection_errors: bool | None = None,
    rootdir: str | None = None,
    collect_only: bool | None = None,
    pyargs: bool | None = None,
    ignore: list[str] | None = None,
    ignore_glob: list[str] | None = None,
    deselect: str | None = None,
    confcutdir: str | None = None,
    noconftest: bool | None = None,
    keep_duplicates: bool | None = None,
    collect_in_virtualenv: bool | None = None,
    import_mode: Literal["prepend", "append", "importlib"] | None = None,
    doctest_modules: bool | None = None,
    doctest_report: Literal["none", "cdiff", "ndiff", "udiff", "only_first_failure"] | None = None,
    doctest_glob: str | None = None,
    doctest_ignore_import_errors: bool | None = None,
    doctest_continue_on_failure: bool | None = None,
    basetemp: str | None = None,
    plugins: list[str] | None = None,
    no_plugins: list[str] | None = None,
    trace_config: bool | None = None,
    debug: str | None = None,
    override_ini: str | None = None,
    assert_mode: str | None = None,
    setup_only: bool | None = None,
    setup_show: bool | None = None,
    setup_plan: bool | None = None,
    log_level: str | None = None,
    log_format: str | None = None,
    log_date_format: str | None = None,
    log_cli_level: tuple[str, str] | None = None,
    log_cli_format: str | None = None,
    log_cli_date_format: str | None = None,
    log_file: str | None = None,
    log_file_level: str | None = None,
    log_file_format: str | None = None,
    log_file_date_format: str | None = None,
    log_auto_indent: str | None = None,
) -> int:
    """Run `pytest`.

    Parameters:
        *paths: Files or directories to select tests from.
        select: Only run tests which match the given substring expression. An expression is a Python evaluatable expression where all names are substring-matched against test names and their parent classes. Example: -k 'test_method or test_other' matches all test functions and classes whose name contains 'test_method' or 'test_other', while -k 'not test_method' matches those that don't contain 'test_method' in their names. -k 'not test_method and not test_other' will eliminate the matches. Additionally keywords are matched to classes and functions containing extra names in their 'extra_keyword_matches' set, as well as functions which have names assigned directly to them. The matching is case-insensitive.
        select_markers: Only run tests matching given mark expression. For example: -m 'mark1 and not mark2'.
        markers: show markers (builtin, plugin and per-project ones).
        exitfirst: Exit instantly on first error or failed test
        fixtures: Show available fixtures, sorted by plugin appearance (fixtures with leading '_' are only shown with '-v')
        fixtures_per_test: Show fixtures per test
        pdb: Start the interactive Python debugger on errors or KeyboardInterrupt
        pdbcls: Specify a custom interactive Python debugger for use with --pdb.For example: --pdbcls IPython.terminal.debugger:TerminalPdb
        trace: Immediately break when running each test
        capture: Per-test capturing method: one of fd|sys|no|tee-sys
        runxfail: Report the results of xfail tests as if they were not marked
        last_failed: Rerun only the tests that failed at the last run (or all if none failed)
        failed_first: Run all tests, but run the last failures first. This may re-order tests and thus lead to repeated fixture setup/teardown.
        new_first: Run tests from new files first, then the rest of the tests sorted by file mtime
        cache_show: Show cache contents, don't perform collection or tests. Optional argument: glob (default: '*').
        cache_clear: Remove all cache contents at start of test run
        last_failed_no_failures: Which tests to run with no previously (known) failures
        stepwise: Exit on test failure and continue from last failing test next time
        stepwise_skip: Ignore the first failing test but stop on the next failing test. Implicitly enables --stepwise.
        durations: Show N slowest setup/test durations (N 0 for all)
        durations_min: Minimal duration in seconds for inclusion in slowest list. Default: 0.005.
        verbose: Increase verbosity
        no_header: Disable header
        no_summary: Disable summary
        quiet: Decrease verbosity
        verbosity: Set verbosity. Default: 0.
        show_extra_summary: Show extra test summary info as specified by chars: (f)ailed, (E)rror, (s)kipped, (x)failed, (X)passed, (p)assed, (P)assed with output, (a)ll except passed (p/P), or (A)ll. (w)arnings are enabled by default (see --disable-warnings), 'N' can be used to reset the list. (default: 'fE').
        disable_pytest_warnings: Disable warnings summary
        showlocals: Show locals in tracebacks (disabled by default)
        no_showlocals: Hide locals in tracebacks (negate --showlocals passed through addopts)
        traceback: Traceback print mode (auto/long/short/line/native/no)
        show_capture: Controls how captured stdout/stderr/log is shown on failed tests. Default: all.
        full_trace: Don't cut any tracebacks (default is to cut)
        color: Color terminal output (yes/no/auto)
        code_highlight: {yes,no} Whether code should be highlighted (only if --color is also enabled). Default: yes.
        pastebin: Send failed|all info to bpaste.net pastebin service
        junit_xml: Create junit-xml style report file at given path
        junit_prefix: Prepend prefix to classnames in junit-xml output
        pythonwarnings: Set which warnings to report, see -W option of Python itself
        maxfail: Exit after first num failures or errors
        strict_config: Any warnings encountered while parsing the `pytest` section of the configuration file raise errors
        strict_markers: Markers not registered in the `markers` section of the configuration file raise errors
        config_file file: Load configuration from `file` instead of trying to locate one of the implicit configuration files
        continue_on_collection_errors: Force test execution even if collection errors occur
        rootdir: Define root directory for tests. Can be relative path: 'root_dir', './root_dir', 'root_dir/another_dir/'; absolute path: '/home/user/root_dir'; path with variables: '$HOME/root_dir'.
        collect_only: Only collect tests, don't execute them
        pyargs: Try to interpret all arguments as Python packages
        ignore: Ignore path during collection (multi-allowed)
        ignore_glob: Ignore path pattern during collection (multi-allowed)
        deselect: Deselect item (via node id prefix) during collection (multi-allowed)
        confcutdir: Only load conftest.py's relative to specified dir
        noconftest: Don't load any conftest.py files
        keep_duplicates: Keep duplicate tests
        collect_in_virtualenv: Don't ignore tests in a local virtualenv directory
        import_mode: Prepend/append to sys.path when importing test modules and conftest files. Default: prepend.
        doctest_modules: Run doctests in all .py modules
        doctest_report: Choose another output format for diffs on doctest failure
        doctest_glob pat: Doctests file matching pattern, default: test*.txt
        doctest_ignore_import_errors: Ignore doctest ImportErrors
        doctest_continue_on_failure: For a given doctest, continue to run after the first failure
        basetemp: Base temporary directory for this test run. (Warning: this directory is removed if it exists.)
        plugins: Early-load given plugin module name or entry point (multi-allowed). To avoid loading of plugins, use the `no:` prefix, e.g. `no:doctest`.
        no_plugins: Early-load given plugin module name or entry point (multi-allowed). To avoid loading of plugins, use the `no:` prefix, e.g. `no:doctest`.
        trace_config: Trace considerations of conftest.py files
        debug: Store internal tracing debug information in this log file. This file is opened with 'w' and truncated as a result, care advised. Default: pytestdebug.log.
        override_ini: Override ini option with "option value" style, e.g. `-o xfail_strict True -o cache_dir cache`.
        assert_mode: Control assertion debugging tools. 'plain' performs no assertion debugging. 'rewrite' (the default) rewrites assert statements in test modules on import to provide assert expression information.
        setup_only: Only setup fixtures, do not execute tests
        setup_show: Show setup of fixtures while executing tests
        setup_plan: Show what fixtures and tests would be executed but don't execute anything
        log_level: Level of messages to catch/display. Not set by default, so it depends on the root/parent log handler's effective level, where it is "WARNING" by default.
        log_format: Log format used by the logging module.
        log_date_format: Log date format used by the logging module.
        log_cli_level: logging level.
        log_cli_format: Log format used by the logging module.
        log_cli_date_format: Log date format used by the logging module.
        log_file: Path to a file when logging will be written to.
        log_file_level: Log file logging level.
        log_file_format: Log format used by the logging module.
        log_file_date_format: Log date format used by the logging module.
        log_auto_indent: Auto-indent multiline messages passed to the logging module. Accepts true|on, false|off or an integer.
    """
    from pytest import main as pytest  # noqa: PT013,PLC0415

    cli_args = list(paths)

    if select:
        cli_args.append("-k")
        cli_args.append(select)

    if select_markers:
        cli_args.append("-m")
        cli_args.append(select_markers)

    if markers:
        cli_args.append("--markers")

    if exitfirst:
        cli_args.append("--exitfirst")

    if fixtures:
        cli_args.append("--fixtures")

    if fixtures_per_test:
        cli_args.append("--fixtures-per-test")

    if pdb:
        cli_args.append("--pdb")

    if pdbcls:
        cli_args.append("--pdbcls")
        cli_args.append(pdbcls)

    if trace:
        cli_args.append("--trace")

    if capture:
        cli_args.append("--capture")

    if runxfail:
        cli_args.append("--runxfail")

    if last_failed:
        cli_args.append("--last-failed")

    if failed_first:
        cli_args.append("--failed-first")

    if new_first:
        cli_args.append("--new-first")

    if cache_show:
        cli_args.append("--cache-show")
        cli_args.append(cache_show)

    if cache_clear:
        cli_args.append("--cache-clear")

    if last_failed_no_failures:
        cli_args.append("--last-failed-no-failures")
        cli_args.append(last_failed_no_failures)

    if stepwise:
        cli_args.append("--stepwise")

    if stepwise_skip:
        cli_args.append("--stepwise-skip")

    if durations:
        cli_args.append("--durations")
        cli_args.append(str(durations))

    if durations_min:
        cli_args.append("--durations-min")
        cli_args.append(str(durations_min))

    if verbose:
        cli_args.append("--verbose")

    if no_header:
        cli_args.append("--no-header")

    if no_summary:
        cli_args.append("--no-summary")

    if quiet:
        cli_args.append("--quiet")

    if verbosity:
        cli_args.append("--verbosity")
        cli_args.append(str(verbosity))

    if show_extra_summary:
        cli_args.append("-r")
        cli_args.append(show_extra_summary)

    if disable_pytest_warnings:
        cli_args.append("--disable-pytest-warnings")

    if showlocals:
        cli_args.append("--showlocals")

    if no_showlocals:
        cli_args.append("--no-showlocals")

    if traceback:
        cli_args.append("--tb")
        cli_args.append(traceback)

    if show_capture:
        cli_args.append("--show-capture")
        cli_args.append(show_capture)

    if full_trace:
        cli_args.append("--full-trace")

    if color:
        cli_args.append("--color")
        cli_args.append(color)

    if code_highlight:
        cli_args.append("--code-highlight")

    if pastebin:
        cli_args.append("--pastebin")
        cli_args.append(pastebin)

    if junit_xml:
        cli_args.append("--junit-xml")
        cli_args.append(junit_xml)

    if junit_prefix:
        cli_args.append("--junit-prefix")
        cli_args.append(junit_prefix)

    if pythonwarnings:
        cli_args.append("--pythonwarnings")
        cli_args.append(pythonwarnings)

    if maxfail:
        cli_args.append("--maxfail")
        cli_args.append(str(maxfail))

    if strict_config:
        cli_args.append("--strict-config")

    if strict_markers:
        cli_args.append("--strict-markers")

    if config_file:
        cli_args.append("-c")
        cli_args.append(config_file)

    if continue_on_collection_errors:
        cli_args.append("--continue-on-collection-errors")

    if rootdir:
        cli_args.append("--rootdir")
        cli_args.append(rootdir)

    if collect_only:
        cli_args.append("--collect-only")

    if pyargs:
        cli_args.append("--pyargs")

    if ignore:
        for ign in ignore:
            cli_args.append("--ignore")
            cli_args.append(ign)

    if ignore_glob:
        for ign_glob in ignore_glob:
            cli_args.append("--ignore-glob")
            cli_args.append(ign_glob)

    if deselect:
        cli_args.append("--deselect")
        cli_args.append(deselect)

    if confcutdir:
        cli_args.append("--confcutdir")
        cli_args.append(confcutdir)

    if noconftest:
        cli_args.append("--noconftest")

    if keep_duplicates:
        cli_args.append("--keep-duplicates")

    if collect_in_virtualenv:
        cli_args.append("--collect-in-virtualenv")

    if import_mode:
        cli_args.append("--import-mode")
        cli_args.append(import_mode)

    if doctest_modules:
        cli_args.append("--doctest-modules")

    if doctest_report:
        cli_args.append("--doctest-report")
        cli_args.append(doctest_report)

    if doctest_glob:
        cli_args.append("--doctest-glob")
        cli_args.append(doctest_glob)

    if doctest_ignore_import_errors:
        cli_args.append("--doctest-ignore-import-errors")

    if doctest_continue_on_failure:
        cli_args.append("--doctest-continue-on-failure")

    if basetemp:
        cli_args.append("--basetemp")
        cli_args.append(basetemp)

    if plugins:
        for plugin in plugins:
            cli_args.append("-p")
            cli_args.append(plugin)

    if no_plugins:
        for no_plugin in no_plugins:
            cli_args.append("-p")
            cli_args.append(f"no:{no_plugin}")

    if trace_config:
        cli_args.append("--trace-config")

    if debug:
        cli_args.append("--debug")
        cli_args.append(debug)

    if override_ini:
        cli_args.append("--override-ini")
        cli_args.append(override_ini)

    if assert_mode:
        cli_args.append("--assert")
        cli_args.append(assert_mode)

    if setup_only:
        cli_args.append("--setup-only")

    if setup_show:
        cli_args.append("--setup-show")

    if setup_plan:
        cli_args.append("--setup-plan")

    if log_level:
        cli_args.append("--log-level")
        cli_args.append(log_level)

    if log_format:
        cli_args.append("--log-format")
        cli_args.append(log_format)

    if log_date_format:
        cli_args.append("--log-date-format")
        cli_args.append(log_date_format)

    if log_cli_level:
        cli_args.append("--log-cli-level")
        cli_args.extend(log_cli_level)

    if log_cli_format:
        cli_args.append("--log-cli-format")
        cli_args.append(log_cli_format)

    if log_cli_date_format:
        cli_args.append("--log-cli-date-format")
        cli_args.append(log_cli_date_format)

    if log_file:
        cli_args.append("--log-file")
        cli_args.append(log_file)

    if log_file_level:
        cli_args.append("--log-file-level")
        cli_args.append(log_file_level)

    if log_file_format:
        cli_args.append("--log-file-format")
        cli_args.append(log_file_format)

    if log_file_date_format:
        cli_args.append("--log-file-date-format")
        cli_args.append(log_file_date_format)

    if log_auto_indent:
        cli_args.append("--log-auto-indent")
        cli_args.append(log_auto_indent)

    return pytest(cli_args)



"""Tests about running duties."""

from __future__ import annotations

from typing import NoReturn
from unittest.mock import NonCallableMock

import pytest

from duty._internal.collection import Collection, Duty
from duty._internal.decorator import duty as decorate
from duty._internal.exceptions import DutyFailure

INTERRUPT_CODE = 130


def test_run_duty() -> None:
    """Run a duty."""
    duty = Duty("name", "description", lambda ctx: 1)
    assert duty.run() is None  # type: ignore[func-returns-value]
    assert duty(duty.context) is None  # type: ignore[func-returns-value]


def test_run_pre_post_duties_lambdas() -> None:
    """Run pre- and post- duties as lambdas."""
    pre_calls = []
    post_calls = []

    duty = Duty(
        "name",
        "description",
        lambda ctx: None,
        pre=[lambda ctx: pre_calls.append(True)],
        post=[lambda ctx: post_calls.append(True)],
    )

    duty.run()

    assert pre_calls[0] is True
    assert post_calls[0] is True


def test_run_pre_post_duties_instances() -> None:
    """Run pre- and post- duties as duties."""
    pre_calls = []
    post_calls = []

    pre_duty = Duty("pre", "", lambda ctx: pre_calls.append(True))
    post_duty = Duty("post", "", lambda ctx: post_calls.append(True))

    duty = Duty(
        name="name",
        description="description",
        function=lambda ctx: None,
        pre=[pre_duty],
        post=[post_duty],
    )

    duty.run()

    assert pre_calls[0] is True
    assert post_calls[0] is True


def test_run_pre_post_duties_refs() -> None:
    """Run pre- and post- duties as duties references."""
    pre_calls = []
    post_calls = []

    collection = Collection()
    collection.add(decorate(lambda ctx: pre_calls.append(True), name="pre"))  # type: ignore[call-overload]
    collection.add(decorate(lambda ctx: post_calls.append(True), name="post"))  # type: ignore[call-overload]

    duty = Duty("name", "description", lambda ctx: None, collection=collection, pre=["pre"], post=["post"])
    duty.run()

    assert pre_calls[0] is True
    assert post_calls[0] is True


def test_dont_run_other_pre_post_duties() -> None:
    """Don't run other types of pre- and post- duties."""
    pre_duty = NonCallableMock()
    post_duty = NonCallableMock()

    duty = Duty("name", "description", lambda ctx: 0, pre=[pre_duty], post=[post_duty])
    duty.run()

    assert not pre_duty.called
    assert not post_duty.called


def test_code_when_keyboard_interrupt() -> None:
    """Return a code 130 on keyboard interruption."""

    def interrupt() -> NoReturn:
        raise KeyboardInterrupt

    with pytest.raises(DutyFailure) as excinfo:
        Duty("name", "description", lambda ctx: ctx.run(interrupt)).run()
    assert excinfo.value.code == INTERRUPT_CODE


def test_dont_raise_duty_failure() -> None:
    """Don't raise a duty failure on success."""
    duty = Duty("n", "d", lambda ctx: ctx.run(lambda: 0))
    assert not duty.run()  # type: ignore[func-returns-value]


def test_cant_find_duty_without_collection() -> None:
    """Check that we can't find a duty with its name without a collection."""
    duty = decorate(lambda ctx: None, name="duty1", post=["duty2"])  # type: ignore[call-overload]
    with pytest.raises(RuntimeError):
        duty.run()



from __future__ import annotations

from typing import Literal

from duty._internal.tools._base import Tool


class griffe(Tool):  # noqa: N801
    """Call [Griffe](https://github.com/mkdocstrings/griffe)."""

    cli_name = "griffe"
    """The name of the executable on PATH."""

    @classmethod
    def check(
        cls,
        package: str,
        *,
        against: str | None = None,
        base_ref: str | None = None,
        color: bool = False,
        verbose: bool = False,
        format: Literal["oneline", "verbose", "markdown", "github"] | None = None,  # noqa: A002
        search: list[str] | None = None,
        sys_path: bool = False,
        find_stubs_packages: bool = False,
        extensions: str | list[str] | None = None,
        inspection: bool | None = None,
        log_level: Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"] | None = None,
        version: bool = False,
        debug_info: bool = False,
    ) -> griffe:
        """Check for API breakages or possible improvements.

        Parameters:
            package: Package to find, load and check, as path.
            against: Older Git reference (commit, branch, tag) to check against. Default: load latest tag.
            base_ref: Git reference (commit, branch, tag) to check. Default: load current code.
            color: Force enable/disable colors in the output.
            verbose: Verbose output.
            format: Output format.
            search: Paths to search packages into.
            sys_path: Whether to append `sys.path` to search paths specified with `-s`.
            find_stubs_packages: Whether to look for stubs-only packages and merge them with concrete ones.
            extensions: A comma-separated list or a JSON list of extensions to load.
            inspection: Whether to disallow or force inspection (dynamic analysis).
                By default, Griffe tries to use static analysis and falls back to dynamic analysis when it can't.
            log_level: Set the log level: `DEBUG`, `INFO`, `WARNING`, `ERROR`, `CRITICAL`.
            version: Show program's version number and exit.
            debug_info: Print debug information.
        """
        cli_args = ["check"]

        if version:
            cli_args.append("--version")

        if debug_info:
            cli_args.append("--debug-info")

        if package:
            cli_args.append(package)

        if against:
            cli_args.append("--against")
            cli_args.append(against)

        if base_ref:
            cli_args.append("--base-ref")
            cli_args.append(base_ref)

        if color is True:
            cli_args.append("--color")
        elif color is False:
            cli_args.append("--no-color")

        if verbose:
            cli_args.append("--verbose")

        if format:
            cli_args.append("--format")
            cli_args.append(format)

        if search:
            for path in search:
                cli_args.append("--search")
                cli_args.append(path)

        if sys_path:
            cli_args.append("--sys-path")

        if find_stubs_packages:
            cli_args.append("--find-stubs-packages")

        if extensions:
            cli_args.append("--extensions")
            if isinstance(extensions, str):
                cli_args.append(extensions)
            else:
                cli_args.append(",".join(extensions))

        if inspection is True:
            cli_args.append("--force-inspection")
        elif inspection is False:
            cli_args.append("--no-inspection")

        if log_level:
            cli_args.append("--log-level")
            cli_args.append(log_level)

        return cls(cli_args)

    @classmethod
    def dump(
        cls,
        *packages: str,
        full: bool = False,
        output: str | None = None,
        docstyle: str | None = None,
        docopts: str | None = None,
        resolve_aliases: bool = False,
        resolve_implicit: bool = False,
        resolve_external: bool | None = None,
        stats: bool = False,
        search: list[str] | None = None,
        sys_path: bool = False,
        find_stubs_packages: bool = False,
        extensions: str | list[str] | None = None,
        inspection: bool | None = None,
        log_level: Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"] | None = None,
        version: bool = False,
        debug_info: bool = False,
    ) -> griffe:
        """Load package-signatures and dump them as JSON.

        Parameters:
            packages: Packages to find, load and dump.
            full: Whether to dump full data in JSON.
            output: Output file. Supports templating to output each package in its own file, with `{package}`.
            docstyle: The docstring style to parse.
            docopts: The options for the docstring parser.
            resolve_aliases: Whether to resolve aliases.
            resolve_implicit: Whether to resolve implicitely exported aliases as well. Aliases are explicitely exported when defined in `__all__`.
            resolve_external: Whether to resolve aliases pointing to external/unknown modules (not loaded directly).
                Default is to resolve only from one module to its private sibling (`ast` -> `_ast`).
            stats: Show statistics at the end.
            search: Paths to search packages into.
            sys_path: Whether to append `sys.path` to search paths specified with `-s`.
            find_stubs_packages: Whether to look for stubs-only packages and merge them with concrete ones.
            extensions: A comma-separated list or a JSON list of extensions to load.
            inspection: Whether to disallow or force inspection (dynamic analysis).
                By default, Griffe tries to use static analysis and falls back to dynamic analysis when it can't.
            log_level: Set the log level: `DEBUG`, `INFO`, `WARNING`, `ERROR`, `CRITICAL`.
            version: Show program's version number and exit.
            debug_info: Print debug information.
        """
        cli_args = ["dump", *packages]

        if version:
            cli_args.append("--version")

        if debug_info:
            cli_args.append("--debug-info")

        if full:
            cli_args.append("--full")

        if output:
            cli_args.append("--output")
            cli_args.append(output)

        if docstyle:
            cli_args.append("--docstyle")
            cli_args.append(docstyle)

        if docopts:
            cli_args.append("--docopts")
            cli_args.append(docopts)

        if resolve_aliases:
            cli_args.append("--resolve-aliases")

        if resolve_implicit:
            cli_args.append("--resolve-implicit")

        if resolve_external is True:
            cli_args.append("--resolve-external")
        elif resolve_external is False:
            cli_args.append("--no-resolve-external")

        if stats:
            cli_args.append("--stats")

        if search:
            for path in search:
                cli_args.append("--search")
                cli_args.append(path)

        if sys_path:
            cli_args.append("--sys-path")

        if find_stubs_packages:
            cli_args.append("--find-stubs-packages")

        if extensions:
            cli_args.append("--extensions")
            if isinstance(extensions, str):
                cli_args.append(extensions)
            else:
                cli_args.append(",".join(extensions))

        if inspection is True:
            cli_args.append("--force-inspection")
        elif inspection is False:
            cli_args.append("--no-inspection")

        if log_level:
            cli_args.append("--log-level")
            cli_args.append(log_level)

        return cls(cli_args)

    def __call__(self) -> int:
        """Run the command.

        Returns:
            The exit code of the command.
        """
        from griffe import main as run_griffe  # noqa: PLC0415

        return run_griffe(self.cli_args)



from __future__ import annotations

from typing import Literal

from duty._internal.tools._base import Tool

_BADGE_STYLE = Literal["flat", "flat-square", "flat-square-modified", "for-the-badge", "plastic", "social"]


class interrogate(Tool):  # noqa: N801
    """Call [Interrogate](https://github.com/econchick/interrogate)."""

    cli_name = "interrogate"
    """The name of the executable on PATH."""

    def __init__(
        self,
        *src: str,
        verbose: int | None = None,
        quiet: bool | None = None,
        fail_under: float | None = None,
        exclude: str | None = None,
        ignore_init_method: bool | None = None,
        ignore_init_module: bool | None = None,
        ignore_magic: bool | None = None,
        ignore_module: bool | None = None,
        ignore_nested_functions: bool | None = None,
        ignore_nested_classes: bool | None = None,
        ignore_private: bool | None = None,
        ignore_property_decorators: bool | None = None,
        ignore_setters: bool | None = None,
        ignore_semiprivate: bool | None = None,
        ignore_regex: str | None = None,
        whitelist_regex: str | None = None,
        output: str | None = None,
        config: str | None = None,
        color: bool | None = None,
        omit_covered_files: bool | None = None,
        generate_badge: str | None = None,
        badge_format: Literal["png", "svg"] | None = None,
        badge_style: _BADGE_STYLE | None = None,
    ) -> None:
        """Run `interrogate`.

        Args:
            src: Format the directories and file paths.
            verbose: Level of verbosity.
            quiet: Do not print output.
            fail_under: Fail when coverage % is less than a given amount.
            exclude: Exclude PATHs of files and/or directories.
            ignore_init_method: Ignore `__init__` method of classes.
            ignore_init_module: Ignore `__init__.py` modules.
            ignore_magic: Ignore all magic methods of classes.
            ignore_module: Ignore module-level docstrings.
            ignore_nested_functions: Ignore nested functions and methods.
            ignore_nested_classes: Ignore nested classes.
            ignore_private: Ignore private classes, methods, and functions starting with two underscores.
            ignore_property_decorators: Ignore methods with property setter/getter decorators.
            ignore_setters: Ignore methods with property setter decorators.
            ignore_semiprivate: Ignore semiprivate classes, methods, and functions starting with a single underscore.
            ignore_regex: Regex identifying class, method, and function names to ignore.
            whitelist_regex: Regex identifying class, method, and function names to include.
            output: Write output to a given FILE.
            config: Read configuration from pyproject.toml or setup.cfg.
            color: Toggle color output on/off when printing to stdout.
            omit_covered_files: Omit reporting files that have 100% documentation coverage.
            generate_badge: Generate a shields.io status badge (an SVG image) in at a given file or directory.
            badge_format: File format for the generated badge.
            badge_style: Desired style of shields.io badge.
        """
        cli_args = list(src)

        if verbose:
            cli_args.append("--verbose")
            cli_args.append(str(verbose))

        if quiet:
            cli_args.append("--quiet")

        if fail_under:
            cli_args.append("--fail-under")
            cli_args.append(str(fail_under))

        if exclude:
            cli_args.append("--exclude")
            cli_args.append(exclude)

        if ignore_init_method:
            cli_args.append("--ignore-init-method")

        if ignore_init_module:
            cli_args.append("--ignore-init-module")

        if ignore_magic:
            cli_args.append("--ignore-magic")

        if ignore_module:
            cli_args.append("--ignore-module")

        if ignore_nested_functions:
            cli_args.append("--ignore-nested-functions")

        if ignore_nested_classes:
            cli_args.append("--ignore-nested-classes")

        if ignore_private:
            cli_args.append("--ignore-private")

        if ignore_property_decorators:
            cli_args.append("--ignore-property-decorators")

        if ignore_setters:
            cli_args.append("--ignore-setters")

        if ignore_semiprivate:
            cli_args.append("--ignore-semiprivate")

        if ignore_regex:
            cli_args.append("--ignore-regex")
            cli_args.append(ignore_regex)

        if whitelist_regex:
            cli_args.append("--whitelist-regex")
            cli_args.append(whitelist_regex)

        if output:
            cli_args.append("--output")
            cli_args.append(output)

        if omit_covered_files:
            cli_args.append("--omit-covered-files")

        if generate_badge:
            cli_args.append("--generate-badge")
            cli_args.append(generate_badge)

        if badge_format:
            cli_args.append("--badge-format")
            cli_args.append(badge_format)

        if badge_style:
            cli_args.append("--badge-style")
            cli_args.append(badge_style)

        if config:
            cli_args.append("--config")
            cli_args.append(config)

        if color is True:
            cli_args.append("--color")
        elif color is False:
            cli_args.append("--no-color")

        super().__init__(cli_args)

    def __call__(self) -> None:
        """Run the command."""
        from interrogate.cli import main as run_interrogate  # noqa: PLC0415

        return run_interrogate(self.cli_args)



# Script to generate the project's credits.

from __future__ import annotations

import os
import sys
from collections import defaultdict
from collections.abc import Iterable
from importlib.metadata import distributions
from itertools import chain
from pathlib import Path
from textwrap import dedent
from typing import Union

from jinja2 import StrictUndefined
from jinja2.sandbox import SandboxedEnvironment
from packaging.requirements import Requirement

# YORE: EOL 3.10: Replace block with line 2.
if sys.version_info >= (3, 11):
    import tomllib
else:
    import tomli as tomllib

project_dir = Path(os.getenv("MKDOCS_CONFIG_DIR", "."))
with project_dir.joinpath("pyproject.toml").open("rb") as pyproject_file:
    pyproject = tomllib.load(pyproject_file)
project = pyproject["project"]
project_name = project["name"]
devdeps = [dep for group in pyproject["dependency-groups"].values() for dep in group if not dep.startswith("-e")]

PackageMetadata = dict[str, Union[str, Iterable[str]]]
Metadata = dict[str, PackageMetadata]


def _merge_fields(metadata: dict) -> PackageMetadata:
    fields = defaultdict(list)
    for header, value in metadata.items():
        fields[header.lower()].append(value.strip())
    return {
        field: value if len(value) > 1 or field in ("classifier", "requires-dist") else value[0]
        for field, value in fields.items()
    }


def _norm_name(name: str) -> str:
    return name.replace("_", "-").replace(".", "-").lower()


def _requirements(deps: list[str]) -> dict[str, Requirement]:
    return {_norm_name((req := Requirement(dep)).name): req for dep in deps}


def _extra_marker(req: Requirement) -> str | None:
    if not req.marker:
        return None
    try:
        return next(marker[2].value for marker in req.marker._markers if getattr(marker[0], "value", None) == "extra")  # type: ignore[union-attr]
    except StopIteration:
        return None


def _get_metadata() -> Metadata:
    metadata = {}
    for pkg in distributions():
        name = _norm_name(pkg.name)  # type: ignore[attr-defined,unused-ignore]
        metadata[name] = _merge_fields(pkg.metadata)  # type: ignore[arg-type]
        metadata[name]["spec"] = set()
        metadata[name]["extras"] = set()
        metadata[name].setdefault("summary", "")
        _set_license(metadata[name])
    return metadata


def _set_license(metadata: PackageMetadata) -> None:
    license_field = metadata.get("license-expression", metadata.get("license", ""))
    license_name = license_field if isinstance(license_field, str) else " + ".join(license_field)
    check_classifiers = license_name in ("UNKNOWN", "Dual License", "") or license_name.count("\n")
    if check_classifiers:
        license_names = []
        for classifier in metadata["classifier"]:
            if classifier.startswith("License ::"):
                license_names.append(classifier.rsplit("::", 1)[1].strip())
        license_name = " + ".join(license_names)
    metadata["license"] = license_name or "?"


def _get_deps(base_deps: dict[str, Requirement], metadata: Metadata) -> Metadata:
    deps = {}
    for dep_name, dep_req in base_deps.items():
        if dep_name not in metadata or dep_name == "duty":
            continue
        metadata[dep_name]["spec"] |= {str(spec) for spec in dep_req.specifier}  # type: ignore[operator]
        metadata[dep_name]["extras"] |= dep_req.extras  # type: ignore[operator]
        deps[dep_name] = metadata[dep_name]

    again = True
    while again:
        again = False
        for pkg_name in metadata:
            if pkg_name in deps:
                for pkg_dependency in metadata[pkg_name].get("requires-dist", []):
                    requirement = Requirement(pkg_dependency)
                    dep_name = _norm_name(requirement.name)
                    extra_marker = _extra_marker(requirement)
                    if (
                        dep_name in metadata
                        and dep_name not in deps
                        and dep_name != project["name"]
                        and (not extra_marker or extra_marker in deps[pkg_name]["extras"])
                    ):
                        metadata[dep_name]["spec"] |= {str(spec) for spec in requirement.specifier}  # type: ignore[operator]
                        deps[dep_name] = metadata[dep_name]
                        again = True

    return deps


def _render_credits() -> str:
    metadata = _get_metadata()
    dev_dependencies = _get_deps(_requirements(devdeps), metadata)
    prod_dependencies = _get_deps(
        _requirements(
            chain(  # type: ignore[arg-type]
                project.get("dependencies", []),
                chain(*project.get("optional-dependencies", {}).values()),
            ),
        ),
        metadata,
    )

    template_data = {
        "project_name": project_name,
        "prod_dependencies": sorted(prod_dependencies.values(), key=lambda dep: str(dep["name"]).lower()),
        "dev_dependencies": sorted(dev_dependencies.values(), key=lambda dep: str(dep["name"]).lower()),
        "more_credits": "http://pawamoy.github.io/credits/",
    }
    template_text = dedent(
        """
        # Credits

        These projects were used to build *{{ project_name }}*. **Thank you!**

        [Python](https://www.python.org/) |
        [uv](https://github.com/astral-sh/uv) |
        [copier-uv](https://github.com/pawamoy/copier-uv)

        {% macro dep_line(dep) -%}
        [{{ dep.name }}](https://pypi.org/project/{{ dep.name }}/) | {{ dep.summary }} | {{ ("`" ~ dep.spec|sort(reverse=True)|join(", ") ~ "`") if dep.spec else "" }} | `{{ dep.version }}` | {{ dep.license }}
        {%- endmacro %}

        {% if prod_dependencies -%}
        ### Runtime dependencies

        Project | Summary | Version (accepted) | Version (last resolved) | License
        ------- | ------- | ------------------ | ----------------------- | -------
        {% for dep in prod_dependencies -%}
        {{ dep_line(dep) }}
        {% endfor %}

        {% endif -%}
        {% if dev_dependencies -%}
        ### Development dependencies

        Project | Summary | Version (accepted) | Version (last resolved) | License
        ------- | ------- | ------------------ | ----------------------- | -------
        {% for dep in dev_dependencies -%}
        {{ dep_line(dep) }}
        {% endfor %}

        {% endif -%}
        {% if more_credits %}**[More credits from the author]({{ more_credits }})**{% endif %}
        """,
    )
    jinja_env = SandboxedEnvironment(undefined=StrictUndefined)
    return jinja_env.from_string(template_text).render(**template_data)


print(_render_credits())



"""Deprecated. Import from `duty` directly."""

# YORE: Bump 2: Remove file.

import warnings
from typing import Any

from duty._internal import exceptions


def __getattr__(name: str) -> Any:
    warnings.warn(
        "Importing from `duty.exceptions` is deprecated. Import from `duty` directly.",
        DeprecationWarning,
        stacklevel=2,
    )
    return getattr(exceptions, name)



"""Deprecated. Use [`duty.tools.safety`][] instead."""

# YORE: Bump 2: Remove file.

import warnings
from typing import Any

from duty._internal.callables import safety as _safety


def __getattr__(name: str) -> Any:
    warnings.warn(
        "Callables are deprecated in favor of tools, use `duty.tools.safety` instead of `duty.callables.safety`.",
        DeprecationWarning,
        stacklevel=2,
    )
    return getattr(_safety, name)



from __future__ import annotations

from duty._internal.tools._autoflake import autoflake
from duty._internal.tools._black import black
from duty._internal.tools._blacken_docs import blacken_docs
from duty._internal.tools._build import build
from duty._internal.tools._coverage import coverage
from duty._internal.tools._flake8 import flake8
from duty._internal.tools._git_changelog import git_changelog
from duty._internal.tools._griffe import griffe
from duty._internal.tools._interrogate import interrogate
from duty._internal.tools._isort import isort
from duty._internal.tools._mkdocs import mkdocs
from duty._internal.tools._mypy import mypy
from duty._internal.tools._pytest import pytest
from duty._internal.tools._ruff import ruff
from duty._internal.tools._safety import safety
from duty._internal.tools._ssort import ssort
from duty._internal.tools._twine import twine
from duty._internal.tools._ty import ty
from duty._internal.tools._yore import yore

__all__ = [
    "autoflake",
    "black",
    "blacken_docs",
    "build",
    "coverage",
    "flake8",
    "git_changelog",
    "griffe",
    "interrogate",
    "isort",
    "mkdocs",
    "mypy",
    "pytest",
    "ruff",
    "safety",
    "ssort",
    "twine",
    "ty",
    "yore",
]



"""Deprecated. Use [`duty.tools.flake8`][] instead."""

# YORE: Bump 2: Remove file.

import warnings
from typing import Any

from duty._internal.callables import flake8 as _flake8


def __getattr__(name: str) -> Any:
    warnings.warn(
        "Callables are deprecated in favor of tools, use `duty.tools.flake8` instead of `duty.callables.flake8`.",
        DeprecationWarning,
        stacklevel=2,
    )
    return getattr(_flake8, name)



"""Our collection of tools."""

import warnings
from typing import Any

from duty._internal.tools import (
    autoflake,
    black,
    blacken_docs,
    build,
    coverage,
    flake8,
    git_changelog,
    griffe,
    interrogate,
    isort,
    mkdocs,
    mypy,
    pytest,
    ruff,
    safety,
    ssort,
    twine,
    ty,
    yore,
)

__all__ = [
    "autoflake",
    "black",
    "blacken_docs",
    "build",
    "coverage",
    "flake8",
    "git_changelog",
    "griffe",
    "interrogate",
    "isort",
    "mkdocs",
    "mypy",
    "pytest",
    "ruff",
    "safety",
    "ssort",
    "twine",
    "ty",
    "yore",
]


# YORE: Bump 2: Remove block.
def __getattr__(name: str) -> Any:
    """Return the tool or lazy object by name."""
    from failprint import lazy  # noqa: F401,PLC0415

    from duty._internal.tools._base import LazyStderr, LazyStdout, Tool  # noqa: F401,PLC0415

    if name in locals():
        warnings.warn(
            f"Importing `{name}` from `duty.tools` is deprecated, import directly from `duty` instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return locals()[name]
    raise AttributeError(f"module '{__name__}' has no attribute '{name}'")



from __future__ import annotations

import inspect
import sys
from copy import deepcopy
from importlib import util as importlib_util
from typing import Any, Callable, ClassVar, Union

from duty._internal.context import Context

DutyListType = list[Union[str, Callable, "Duty"]]
"""Type of a list of duties, which can be a list of strings, callables, or Duty instances."""
default_duties_file = "duties.py"
"""Default path to the duties file, relative to the current working directory."""


class Duty:
    """The main duty class."""

    default_options: ClassVar[dict[str, Any]] = {}
    """Default options used to create the context instance."""

    def __init__(
        self,
        name: str,
        description: str,
        function: Callable,
        collection: Collection | None = None,
        aliases: set | None = None,
        pre: DutyListType | None = None,
        post: DutyListType | None = None,
        opts: dict[str, Any] | None = None,
    ) -> None:
        """Initialize the duty.

        Parameters:
            name: The duty name.
            description: The duty description.
            function: The duty function.
            collection: The collection on which to attach this duty.
            aliases: A list of aliases for this duty.
            pre: A list of duties to run before this one.
            post: A list of duties to run after this one.
            opts: Options used to create the context instance.
        """
        self.name = name
        """The duty name."""
        self.description = description
        """The duty description."""
        self.function = function
        """The duty function."""
        self.aliases = aliases or set()
        """A set of aliases for this duty."""
        self.pre = pre or []
        """A list of duties to run before this one."""
        self.post = post or []
        """A list of duties to run after this one."""
        self.options = opts or self.default_options
        """Options used to create the context instance."""
        self.options_override: dict = {}
        """Options that override `run` and `@duty` options."""

        self.collection: Collection | None = None
        """The collection on which this duty is attached."""
        if collection:
            collection.add(self)

    @property
    def context(self) -> Context:
        """Return a new context instance.

        Returns:
            A new context instance.
        """
        return Context(self.options, self.options_override)

    def run(self, *args: Any, **kwargs: Any) -> None:
        """Run the duty.

        This is just a shortcut for `duty(duty.context, *args, **kwargs)`.

        Parameters:
            args: Positional arguments passed to the function.
            kwargs: Keyword arguments passed to the function.
        """
        self(self.context, *args, **kwargs)

    def run_duties(self, context: Context, duties_list: DutyListType) -> None:
        """Run a list of duties.

        Parameters:
            context: The context to use.
            duties_list: The list of duties to run.

        Raises:
            RuntimeError: When a duty name is given to pre or post duties.
                Indeed, without a parent collection, it is impossible
                to find another duty by its name.
        """
        for duty_item in duties_list:
            if callable(duty_item):
                # Item is a proper duty, or a callable: run it.
                duty_item(context)
            elif isinstance(duty_item, str):
                # Item is a reference to a duty.
                if self.collection is None:
                    raise RuntimeError(f"Can't find duty by name without a collection ({duty_item})")
                # Get the duty and run it.
                self.collection.get(duty_item)(context)

    def __call__(self, context: Context, *args: Any, **kwargs: Any) -> None:
        """Run the duty function.

        Parameters:
            context: The context to use.
            args: Positional arguments passed to the function.
            kwargs: Keyword arguments passed to the function.
        """
        self.run_duties(context, self.pre)
        self.function(context, *args, **kwargs)
        self.run_duties(context, self.post)


class Collection:
    """A collection of duties.

    Attributes:
        path: The path to the duties file.
        duties: The list of duties.
        aliases: A dictionary of aliases pointing to their respective duties.
    """

    def __init__(self, path: str = default_duties_file) -> None:
        """Initialize the collection.

        Parameters:
            path: The path to the duties file.
        """
        self.path = path
        """The path to the duties file."""
        self.duties: dict[str, Duty] = {}
        """The list of duties."""
        self.aliases: dict[str, Duty] = {}
        """A dictionary of aliases pointing to their respective duties."""

    def clear(self) -> None:
        """Clear the collection."""
        self.duties.clear()
        self.aliases.clear()

    def names(self) -> list[str]:
        """Return the list of duties names and aliases.

        Returns:
            The list of duties names and aliases.
        """
        return list(self.duties.keys()) + list(self.aliases.keys())

    def completion_candidates(self, args: tuple[str, ...]) -> list[str]:
        """Find shell completion candidates within this collection.

        Returns:
            The list of shell completion candidates, sorted alphabetically.
        """
        # Find last duty name in args.
        name = None
        names = set(self.names())
        for arg in reversed(args):
            if arg in names:
                name = arg
                break

        completion_names = sorted(names)

        # If no duty found, return names.
        if name is None:
            return completion_names

        params = [
            f"{param.name}="
            for param in inspect.signature(self.get(name).function).parameters.values()
            if param.kind is not param.VAR_POSITIONAL
        ][1:]

        # If duty found, return names *and* duty parameters.
        return completion_names + sorted(params)

    def get(self, name_or_alias: str) -> Duty:
        """Get a duty by its name or alias.

        Parameters:
            name_or_alias: The name or alias of the duty.

        Returns:
            A duty.
        """
        try:
            return self.duties[name_or_alias]
        except KeyError:
            return self.aliases[name_or_alias]

    def format_help(self) -> str:
        """Format a message listing the duties.

        Returns:
            A string listing the duties and their summary.
        """
        lines = []
        # 20 makes the summary aligned with options description
        longest_name = max(*(len(name) for name in self.duties), 20)
        for name, duty in self.duties.items():
            description = duty.description.split("\n")[0]
            lines.append(f"{name:{longest_name}}  {description}")
        return "\n".join(lines)

    def add(self, duty: Duty) -> None:
        """Add a duty to the collection.

        Parameters:
            duty: The duty to add.
        """
        if duty.collection is not None:
            # we must copy the duty to be able to add it
            # in multiple collections
            duty = deepcopy(duty)
        duty.collection = self
        self.duties[duty.name] = duty
        for alias in duty.aliases:
            self.aliases[alias] = duty

    def load(self, path: str | None = None) -> None:
        """Load duties from a Python file.

        Parameters:
            path: The path to the Python file to load.
                Uses the collection's path by default.
        """
        path = path or self.path
        spec = importlib_util.spec_from_file_location("duty.duties", path)
        if spec:
            duties = importlib_util.module_from_spec(spec)
            sys.modules["duty.duties"] = duties
            spec.loader.exec_module(duties)  # type: ignore[union-attr]
            declared_duties = inspect.getmembers(duties, lambda member: isinstance(member, Duty))
            for _, duty in declared_duties:
                self.add(duty)



from __future__ import annotations

from duty._internal.tools._base import Tool


class mkdocs(Tool):  # noqa: N801
    """Call [MkDocs](https://github.com/mkdocs/mkdocs)."""

    cli_name = "mkdocs"
    """The name of the executable on PATH."""

    @classmethod
    def build(
        cls,
        *,
        config_file: str | None = None,
        clean: bool | None = None,
        strict: bool | None = None,
        theme: str | None = None,
        directory_urls: bool | None = None,
        site_dir: str | None = None,
        quiet: bool = False,
        verbose: bool = False,
    ) -> mkdocs:
        """Build the MkDocs documentation.

        Parameters:
            config_file: Provide a specific MkDocs config.
            clean: Remove old files from the site_dir before building (the default).
            strict: Enable strict mode. This will cause MkDocs to abort the build on any warnings.
            theme: The theme to use when building your documentation.
            directory_urls: Use directory URLs when building pages (the default).
            site_dir: The directory to output the result of the documentation build.
            quiet: Silence warnings.
            verbose: Enable verbose output.
        """
        cli_args = ["build"]

        if clean is True:
            cli_args.append("--clean")
        elif clean is False:
            cli_args.append("--dirty")

        if config_file:
            cli_args.append("--config-file")
            cli_args.append(config_file)

        if strict is True:
            cli_args.append("--strict")

        if theme:
            cli_args.append("--theme")
            cli_args.append(theme)

        if directory_urls is True:
            cli_args.append("--use-directory-urls")
        elif directory_urls is False:
            cli_args.append("--no-directory-urls")

        if site_dir:
            cli_args.append("--site_dir")
            cli_args.append(site_dir)

        if quiet and "-q" not in cli_args:
            cli_args.append("--quiet")

        if verbose and "-v" not in cli_args:
            cli_args.append("--verbose")

        return cls(cli_args)

    @classmethod
    def gh_deploy(
        cls,
        *,
        config_file: str | None = None,
        clean: bool | None = None,
        message: str | None = None,
        remote_branch: str | None = None,
        remote_name: str | None = None,
        force: bool | None = None,
        no_history: bool | None = None,
        ignore_version: bool | None = None,
        shell: bool | None = None,
        strict: bool | None = None,
        theme: str | None = None,
        directory_urls: bool | None = None,
        site_dir: str | None = None,
        quiet: bool = False,
        verbose: bool = False,
    ) -> mkdocs:
        """Deploy your documentation to GitHub Pages.

        Parameters:
            config_file: Provide a specific MkDocs config.
            clean: Remove old files from the site_dir before building (the default).
            message: A commit message to use when committing to the GitHub Pages remote branch.
                Commit {sha} and MkDocs {version} are available as expansions.
            remote_branch: The remote branch to commit to for GitHub Pages. This overrides the value specified in config.
            remote_name: The remote name to commit to for GitHub Pages. This overrides the value specified in config
            force: Force the push to the repository.
            no_history: Replace the whole Git history with one new commit.
            ignore_version: Ignore check that build is not being deployed with an older version of MkDocs.
            shell: Use the shell when invoking Git.
            strict: Enable strict mode. This will cause MkDocs to abort the build on any warnings.
            theme: The theme to use when building your documentation.
            directory_urls: Use directory URLs when building pages (the default).
            site_dir: The directory to output the result of the documentation build.
            quiet: Silence warnings.
            verbose: Enable verbose output.
        """
        cli_args = ["gh-deploy"]

        if clean is True:
            cli_args.append("--clean")
        elif clean is False:
            cli_args.append("--dirty")

        if message:
            cli_args.append("--message")
            cli_args.append(message)

        if remote_branch:
            cli_args.append("--remote-branch")
            cli_args.append(remote_branch)

        if remote_name:
            cli_args.append("--remote-name")
            cli_args.append(remote_name)

        if force:
            cli_args.append("--force")

        if no_history:
            cli_args.append("--no-history")

        if ignore_version:
            cli_args.append("--ignore-version")

        if shell:
            cli_args.append("--shell")

        if config_file:
            cli_args.append("--config-file")
            cli_args.append(config_file)

        if strict is True:
            cli_args.append("--strict")

        if theme:
            cli_args.append("--theme")
            cli_args.append(theme)

        if directory_urls is True:
            cli_args.append("--use-directory-urls")
        elif directory_urls is False:
            cli_args.append("--no-directory-urls")

        if site_dir:
            cli_args.append("--site_dir")
            cli_args.append(site_dir)

        if quiet and "-q" not in cli_args:
            cli_args.append("--quiet")

        if verbose and "-v" not in cli_args:
            cli_args.append("--verbose")

        return cls(cli_args)

    @classmethod
    def new(cls, project_directory: str, *, quiet: bool = False, verbose: bool = False) -> mkdocs:
        """Create a new MkDocs project.

        Parameters:
            project_directory: Where to create the project.
            quiet: Silence warnings.
            verbose: Enable verbose output.
        """
        cli_args = ["new", project_directory]

        if quiet and "-q" not in cli_args:
            cli_args.append("--quiet")

        if verbose and "-v" not in cli_args:
            cli_args.append("--verbose")

        return cls(cli_args)

    @classmethod
    def serve(
        cls,
        *,
        config_file: str | None = None,
        dev_addr: str | None = None,
        livereload: bool | None = None,
        dirtyreload: bool | None = None,
        watch_theme: bool | None = None,
        watch: list[str] | None = None,
        strict: bool | None = None,
        theme: str | None = None,
        directory_urls: bool | None = None,
        quiet: bool = False,
        verbose: bool = False,
    ) -> mkdocs:
        """Run the builtin development server.

        Parameters:
            config_file: Provide a specific MkDocs config.
            dev_addr: IP address and port to serve documentation locally (default: localhost:8000).
            livereload: Enable/disable the live reloading in the development server.
            dirtyreload: nable the live reloading in the development server, but only re-build files that have changed.
            watch_theme: Include the theme in list of files to watch for live reloading. Ignored when live reload is not used.
            watch: Directories or files to watch for live reloading.
            strict: Enable strict mode. This will cause MkDocs to abort the build on any warnings.
            theme: The theme to use when building your documentation.
            directory_urls: Use directory URLs when building pages (the default).
            quiet: Silence warnings.
            verbose: Enable verbose output.
        """
        cli_args = ["serve"]

        if dev_addr:
            cli_args.append("--dev-addr")
            cli_args.append(dev_addr)

        if livereload is True:
            cli_args.append("--livereload")
        elif livereload is False:
            cli_args.append("--no-livereload")

        if dirtyreload:
            cli_args.append("--dirtyreload")

        if watch_theme:
            cli_args.append("--watch-theme")

        if watch:
            for path in watch:
                cli_args.append("--watch")
                cli_args.append(path)

        if config_file:
            cli_args.append("--config-file")
            cli_args.append(config_file)

        if strict is True:
            cli_args.append("--strict")

        if theme:
            cli_args.append("--theme")
            cli_args.append(theme)

        if directory_urls is True:
            cli_args.append("--use-directory-urls")
        elif directory_urls is False:
            cli_args.append("--no-directory-urls")

        if quiet and "-q" not in cli_args:
            cli_args.append("--quiet")

        if verbose and "-v" not in cli_args:
            cli_args.append("--verbose")

        return cls(cli_args)

    def __call__(self) -> int:
        """Run the command.

        Returns:
            The exit code of the command.
        """
        from mkdocs.__main__ import cli as run_mkdocs  # noqa: PLC0415

        return run_mkdocs(self.cli_args)



from __future__ import annotations

import sys

from duty._internal.tools._base import Tool


class ssort(Tool):  # noqa: N801
    """Call [ssort](https://github.com/bwhmather/ssort)."""

    cli_name = "ssort"
    """The name of the executable on PATH."""

    def __init__(
        self,
        *files: str,
        diff: bool | None = None,
        check: bool | None = None,
    ) -> None:
        """Run `ssort`.

        Parameters:
            *files: Files to format.
            diff: Prints a diff of all changes ssort would make to a file.
            check: Check the file for unsorted statements. Returns 0 if nothing needs to be changed. Otherwise returns 1.
        """
        cli_args = list(files)

        if diff:
            cli_args.append("--diff")

        if check:
            cli_args.append("--check")

    def __call__(self) -> None:
        """Run the command.

        Returns:
            The exit code of the command.
        """
        from ssort._main import main as run_ssort  # noqa: PLC0415

        old_sys_argv = sys.argv
        sys.argv = ["ssort", *self.cli_args]
        try:
            run_ssort()
        finally:
            sys.argv = old_sys_argv



# YORE: Bump 2: Remove file.

from __future__ import annotations

from typing import Literal

from failprint import lazy


def run(*args: str, version: bool = False, debug_info: bool = False) -> None:
    """Run `griffe`.

    Parameters:
        *args: CLI arguments.
        version: Show program's version number and exit.
        debug_info: Print debug information.
    """
    from griffe.cli import main as griffe  # noqa: PLC0415

    cli_args = []

    # --version and --debug-info must appear first.
    if version:
        cli_args.append("--version")

    if debug_info:
        cli_args.append("--debug-info")

    cli_args.extend(args)

    griffe(cli_args)


@lazy(name="griffe.check")
def check(
    package: str,
    *,
    against: str | None = None,
    base_ref: str | None = None,
    color: bool = False,
    verbose: bool = False,
    format: Literal["oneline", "verbose", "markdown", "github"] | None = None,  # noqa: A002
    search: list[str] | None = None,
    sys_path: bool = False,
    find_stubs_packages: bool = False,
    extensions: str | list[str] | None = None,
    inspection: bool | None = None,
    log_level: Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"] | None = None,
    version: bool = False,
    debug_info: bool = False,
) -> None:
    """Check for API breakages or possible improvements.

    Parameters:
        package: Package to find, load and check, as path.
        against: Older Git reference (commit, branch, tag) to check against. Default: load latest tag.
        base_ref: Git reference (commit, branch, tag) to check. Default: load current code.
        color: Force enable/disable colors in the output.
        verbose: Verbose output.
        format: Output format.
        search: Paths to search packages into.
        sys_path: Whether to append `sys.path` to search paths specified with `-s`.
        find_stubs_packages: Whether to look for stubs-only packages and merge them with concrete ones.
        extensions: A comma-separated list or a JSON list of extensions to load.
        inspection: Whether to disallow or force inspection (dynamic analysis).
            By default, Griffe tries to use static analysis and falls back to dynamic analysis when it can't.
        log_level: Set the log level: `DEBUG`, `INFO`, `WARNING`, `ERROR`, `CRITICAL`.
        version: Show program's version number and exit.
        debug_info: Print debug information.
    """
    cli_args = []

    if package:
        cli_args.append(package)

    if against:
        cli_args.append("--against")
        cli_args.append(against)

    if base_ref:
        cli_args.append("--base-ref")
        cli_args.append(base_ref)

    if color is True:
        cli_args.append("--color")
    elif color is False:
        cli_args.append("--no-color")

    if verbose:
        cli_args.append("--verbose")

    if format:
        cli_args.append("--format")
        cli_args.append(format)

    if search:
        for path in search:
            cli_args.append("--search")
            cli_args.append(path)

    if sys_path:
        cli_args.append("--sys-path")

    if find_stubs_packages:
        cli_args.append("--find-stubs-packages")

    if extensions:
        cli_args.append("--extensions")
        if isinstance(extensions, str):
            cli_args.append(extensions)
        else:
            cli_args.append(",".join(extensions))

    if inspection is True:
        cli_args.append("--force-inspection")
    elif inspection is False:
        cli_args.append("--no-inspection")

    if log_level:
        cli_args.append("--log-level")
        cli_args.append(log_level)

    run("check", *cli_args, version=version, debug_info=debug_info)


@lazy(name="griffe.dump")
def dump(
    *packages: str,
    full: bool = False,
    output: str | None = None,
    docstyle: str | None = None,
    docopts: str | None = None,
    resolve_aliases: bool = False,
    resolve_implicit: bool = False,
    resolve_external: bool | None = None,
    stats: bool = False,
    search: list[str] | None = None,
    sys_path: bool = False,
    find_stubs_packages: bool = False,
    extensions: str | list[str] | None = None,
    inspection: bool | None = None,
    log_level: Literal["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"] | None = None,
    version: bool = False,
    debug_info: bool = False,
) -> None:
    """Load package-signatures and dump them as JSON.

    Parameters:
        packages: Packages to find, load and dump.
        full: Whether to dump full data in JSON.
        output: Output file. Supports templating to output each package in its own file, with `{package}`.
        docstyle: The docstring style to parse.
        docopts: The options for the docstring parser.
        resolve_aliases: Whether to resolve aliases.
        resolve_implicit: Whether to resolve implicitely exported aliases as well. Aliases are explicitely exported when defined in `__all__`.
        resolve_external: Whether to resolve aliases pointing to external/unknown modules (not loaded directly).
            Default is to resolve only from one module to its private sibling (`ast` -> `_ast`).
        stats: Show statistics at the end.
        search: Paths to search packages into.
        sys_path: Whether to append `sys.path` to search paths specified with `-s`.
        find_stubs_packages: Whether to look for stubs-only packages and merge them with concrete ones.
        extensions: A comma-separated list or a JSON list of extensions to load.
        inspection: Whether to disallow or force inspection (dynamic analysis).
            By default, Griffe tries to use static analysis and falls back to dynamic analysis when it can't.
        log_level: Set the log level: `DEBUG`, `INFO`, `WARNING`, `ERROR`, `CRITICAL`.
        version: Show program's version number and exit.
        debug_info: Print debug information.
    """
    cli_args = list(packages)

    if full:
        cli_args.append("--full")

    if output:
        cli_args.append("--output")
        cli_args.append(output)

    if docstyle:
        cli_args.append("--docstyle")
        cli_args.append(docstyle)

    if docopts:
        cli_args.append("--docopts")
        cli_args.append(docopts)

    if resolve_aliases:
        cli_args.append("--resolve-aliases")

    if resolve_implicit:
        cli_args.append("--resolve-implicit")

    if resolve_external is True:
        cli_args.append("--resolve-external")
    elif resolve_external is False:
        cli_args.append("--no-resolve-external")

    if stats:
        cli_args.append("--stats")

    if search:
        for path in search:
            cli_args.append("--search")
            cli_args.append(path)

    if sys_path:
        cli_args.append("--sys-path")

    if find_stubs_packages:
        cli_args.append("--find-stubs-packages")

    if extensions:
        cli_args.append("--extensions")
        if isinstance(extensions, str):
            cli_args.append(extensions)
        else:
            cli_args.append(",".join(extensions))

    if inspection is True:
        cli_args.append("--force-inspection")
    elif inspection is False:
        cli_args.append("--no-inspection")

    if log_level:
        cli_args.append("--log-level")
        cli_args.append(log_level)

    run("check", *cli_args, version=version, debug_info=debug_info)



from __future__ import annotations

from typing import Literal

from duty._internal.tools._base import LazyStderr, LazyStdout, Tool


class mypy(Tool):  # noqa: N801
    """Call [Mypy](https://github.com/python/mypy)."""

    cli_name = "mypy"
    """The name of the executable on PATH."""

    def __init__(
        self,
        *paths: str,
        config_file: str | None = None,
        enable_incomplete_feature: bool | None = None,
        verbose: bool | None = None,
        warn_unused_configs: bool | None = None,
        no_namespace_packages: bool | None = None,
        ignore_missing_imports: bool | None = None,
        follow_imports: Literal["normal", "silent", "skip", "error"] | None = None,
        python_executable: str | None = None,
        no_site_packages: bool | None = None,
        no_silence_site_packages: bool | None = None,
        python_version: str | None = None,
        py2: bool | None = None,
        platform: str | None = None,
        always_true: list[str] | None = None,
        always_false: list[str] | None = None,
        disallow_any_unimported: bool | None = None,
        disallow_any_expr: bool | None = None,
        disallow_any_decorated: bool | None = None,
        disallow_any_explicit: bool | None = None,
        disallow_any_generics: bool | None = None,
        disallow_subclassing_any: bool | None = None,
        disallow_untyped_calls: bool | None = None,
        disallow_untyped_defs: bool | None = None,
        disallow_incomplete_defs: bool | None = None,
        check_untyped_defs: bool | None = None,
        disallow_untyped_decorators: bool | None = None,
        implicit_optional: bool | None = None,
        no_strict_optional: bool | None = None,
        warn_redundant_casts: bool | None = None,
        warn_unused_ignores: bool | None = None,
        no_warn_no_return: bool | None = None,
        warn_return_any: bool | None = None,
        warn_unreachable: bool | None = None,
        allow_untyped_globals: bool | None = None,
        allow_redefinition: bool | None = None,
        no_implicit_reexport: bool | None = None,
        strict_equality: bool | None = None,
        strict_concatenate: bool | None = None,
        strict: bool | None = None,
        disable_error_code: str | None = None,
        enable_error_code: str | None = None,
        show_error_context: bool | None = None,
        show_column_numbers: bool | None = None,
        show_error_end: bool | None = None,
        hide_error_codes: bool | None = None,
        pretty: bool | None = None,
        no_color_output: bool | None = None,
        no_error_summary: bool | None = None,
        show_absolute_path: bool | None = None,
        no_incremental: bool | None = None,
        cache_dir: str | None = None,
        sqlite_cache: bool | None = None,
        cache_fine_grained: bool | None = None,
        skip_version_check: bool | None = None,
        skip_cache_mtime_checks: bool | None = None,
        pdb: bool | None = None,
        show_traceback: bool | None = None,
        raise_exceptions: bool | None = None,
        custom_typing_module: str | None = None,
        disable_recursive_aliases: bool | None = None,
        custom_typeshed_dir: str | None = None,
        warn_incomplete_stub: bool | None = None,
        shadow_file: tuple[str, str] | None = None,
        any_exprs_report: str | None = None,
        cobertura_xml_report: str | None = None,
        html_report: str | None = None,
        linecount_report: str | None = None,
        linecoverage_report: str | None = None,
        lineprecision_report: str | None = None,
        txt_report: str | None = None,
        xml_report: str | None = None,
        xslt_html_report: str | None = None,
        xslt_txt_report: str | None = None,
        junit_xml: str | None = None,
        find_occurrences: str | None = None,
        scripts_are_modules: bool | None = None,
        install_types: bool | None = None,
        non_interactive: bool | None = None,
        explicit_package_bases: bool | None = None,
        exclude: str | None = None,
        module: str | None = None,
        package: str | None = None,
        command: str | None = None,
    ) -> None:
        """Run mypy.

        Parameters:
            *paths: Path to scan.
            config_file: Configuration file, must have a [mypy] section (defaults to mypy.ini, .mypy.ini,
            enable_incomplete_feature: Enable support of incomplete/experimental features for early preview.
            verbose: More verbose messages.
                pyproject.toml, setup.cfg, /home/pawamoy/.config/mypy/config, ~/.config/mypy/config, ~/.mypy.ini).
            warn_unused_configs: Warn about unused '[mypy-<pattern>]' or '[[tool.mypy.overrides]]' config sections
                (inverse: --no-warn-unused-configs).
            no_namespace_packages: Support namespace packages (PEP 420, __init__.py-less) (inverse: --namespace-packages).
            ignore_missing_imports: Silently ignore imports of missing modules.
            follow_imports: How to treat imports (default normal).
            python_executable: Python executable used for finding PEP 561 compliant installed packages and stubs.
            no_site_packages: Do not search for installed PEP 561 compliant packages.
            no_silence_site_packages: Do not silence errors in PEP 561 compliant installed packages.
            python_version: Type check code assuming it will be running on Python x.y.
            py2: Use Python 2 mode (same as --python-version 2.7).
            platform: Type check special-cased code for the given OS platform (defaults to sys.platform).
            always_true: Additional variable to be considered True (may be repeated).
            always_false: Additional variable to be considered False (may be repeated).
            disallow_any_unimported: Disallow Any types resulting from unfollowed imports.
            disallow_any_expr: Disallow all expressions that have type Any.
            disallow_any_decorated: Disallow functions that have Any in their signature after decorator transformation.
            disallow_any_explicit: Disallow explicit Any in type positions.
            disallow_any_generics: Disallow usage of generic types that do not specify explicit type parameters
                (inverse: --allow-any-generics).
            disallow_subclassing_any: Disallow subclassing values of type 'Any' when defining classes
                (inverse: --allow-subclassing-any).
            disallow_untyped_calls: Disallow calling functions without type annotations from functions with type annotations
                (inverse: --allow-untyped-calls).
            disallow_untyped_defs: Disallow defining functions without type annotations or with incomplete type annotations
                (inverse: --allow-untyped-defs).
            disallow_incomplete_defs: Disallow defining functions with incomplete type annotations
                (inverse: --allow-incomplete-defs).
            check_untyped_defs: Type check the interior of functions without type annotations
                (inverse: --no-check-untyped-defs).
            disallow_untyped_decorators: Disallow decorating typed functions with untyped decorators
                (inverse: --allow-untyped-decorators).
            implicit_optional: Assume arguments with default values of None are Optional(inverse: --no-implicit-optional).
            no_strict_optional: Disable strict Optional checks (inverse: --strict-optional).
            warn_redundant_casts: Warn about casting an expression to its inferred type (inverse: --no-warn-redundant-casts).
            warn_unused_ignores: Warn about unneeded '# type: ignore' comments (inverse: --no-warn-unused-ignores).
            no_warn_no_return: Do not warn about functions that end without returning (inverse: --warn-no-return).
            warn_return_any: Warn about returning values of type Any from non-Any typed functions (inverse: --no-warn-return-any).
            warn_unreachable: Warn about statements or expressions inferred to be unreachable (inverse: --no-warn-unreachable).
            allow_untyped_globals: Suppress toplevel errors caused by missing annotations (inverse: --disallow-untyped-globals).
            allow_redefinition: Allow unconditional variable redefinition with a new type (inverse: --disallow-redefinition).
            no_implicit_reexport: Treat imports as private unless aliased (inverse: --implicit-reexport).
            strict_equality: Prohibit equality, identity, and container checks for non-overlapping types
                (inverse: --no-strict-equality).
            strict_concatenate: Make arguments prepended via Concatenate be truly positional-only (inverse: --no-strict-concatenate).
            strict: Strict mode; enables the following flags: --warn-unused-configs, --disallow-any-generics,
                --disallow-subclassing-any, --disallow-untyped-calls, --disallow-untyped-defs, --disallow-incomplete-defs,
                --check-untyped-defs, --disallow-untyped-decorators, --warn-redundant-casts, --warn-unused-ignores,
                --warn-return-any, --no-implicit-reexport, --strict-equality, --strict-concatenate.
            disable_error_code: Disable a specific error code.
            enable_error_code: Enable a specific error code.
            show_error_context: Precede errors with "note:" messages explaining context (inverse: --hide-error-context).
            show_column_numbers: Show column numbers in error messages (inverse: --hide-column-numbers).
            show_error_end: Show end line/end column numbers in error messages. This implies --show-column-numbers
                (inverse: --hide-error-end).
            hide_error_codes: Hide error codes in error messages (inverse: --show-error-codes).
            pretty: Use visually nicer output in error messages: Use soft word wrap, show source code snippets,
                and show error location markers (inverse: --no-pretty).
            no_color_output: Do not colorize error messages (inverse: --color-output).
            no_error_summary: Do not show error stats summary (inverse: --error-summary).
            show_absolute_path: Show absolute paths to files (inverse: --hide-absolute-path).
            no_incremental: Disable module cache (inverse: --incremental).
            cache_dir: Store module cache info in the given folder in incremental mode (defaults to '.mypy_cache').
            sqlite_cache: Use a sqlite database to store the cache (inverse: --no-sqlite-cache).
            cache_fine_grained: Include fine-grained dependency information in the cache for the mypy daemon.
            skip_version_check: Allow using cache written by older mypy version.
            skip_cache_mtime_checks: Skip cache internal consistency checks based on mtime.
            pdb: Invoke pdb on fatal error.
            show_traceback: Show traceback on fatal error.
            raise_exceptions: Raise exception on fatal error.
            custom_typing_module: Use a custom typing module.
            disable_recursive_aliases: Disable experimental support for recursive type aliases.
            custom_typeshed_dir: Use the custom typeshed in DIR.
            warn_incomplete_stub: Warn if missing type annotation in typeshed, only relevant with --disallow-untyped-defs
                or --disallow-incomplete-defs enabled (inverse: --no-warn-incomplete-stub).
            shadow_file: When encountering SOURCE_FILE, read and type check the contents of SHADOW_FILE instead..
            any_exprs_report: Report any expression.
            cobertura_xml_report: Report Cobertura.
            html_report: Report HTML.
            linecount_report: Report line count.
            linecoverage_report: Report line coverage.
            lineprecision_report: Report line precision.
            txt_report: Report text.
            xml_report: Report XML.
            xslt_html_report: Report XLST HTML.
            xslt_txt_report: Report XLST text.
            junit_xml: Write junit.xml to the given file.
            find_occurrences: Print out all usages of a class member (experimental).
            scripts_are_modules: Script x becomes module x instead of __main__.
            install_types: Install detected missing library stub packages using pip (inverse: --no-install-types).
            non_interactive: Install stubs without asking for confirmation and hide errors, with --install-types
                (inverse: --interactive).
            explicit_package_bases: Use current directory and MYPYPATH to determine module names of files passed
                (inverse: --no-explicit-package-bases).
            exclude: Regular expression to match file names, directory names or paths which mypy should ignore while
                recursively discovering files to check, e.g. --exclude '/setup\\.py$'.
                May be specified more than once, eg. --exclude a --exclude b.
            module: Type-check module; can repeat for more modules.
            package: Type-check package recursively; can be repeated.
            command: Type-check program passed in as string.
        """  # noqa: D301
        cli_args = list(paths)

        if enable_incomplete_feature:
            cli_args.append("--enable-incomplete-feature")

        if verbose:
            cli_args.append("--verbose")

        if config_file:
            cli_args.append("--config-file")
            cli_args.append(config_file)

        if warn_unused_configs:
            cli_args.append("--warn-unused-configs")

        if no_namespace_packages:
            cli_args.append("--no-namespace-packages")

        if ignore_missing_imports:
            cli_args.append("--ignore-missing-imports")

        if follow_imports:
            cli_args.append("--follow-imports")
            cli_args.append(follow_imports)

        if python_executable:
            cli_args.append("--python-executable")
            cli_args.append(python_executable)

        if no_site_packages:
            cli_args.append("--no-site-packages")

        if no_silence_site_packages:
            cli_args.append("--no-silence-site-packages")

        if python_version:
            cli_args.append("--python-version")
            cli_args.append(python_version)

        if py2:
            cli_args.append("--py2")

        if platform:
            cli_args.append("--platform")
            cli_args.append(platform)

        if always_true:
            for posarg in always_true:
                cli_args.append("--always-true")
                cli_args.append(posarg)

        if always_false:
            for posarg in always_false:
                cli_args.append("--always-false")
                cli_args.append(posarg)

        if disallow_any_unimported:
            cli_args.append("--disallow-any-unimported")

        if disallow_any_expr:
            cli_args.append("--disallow-any-expr")

        if disallow_any_decorated:
            cli_args.append("--disallow-any-decorated")

        if disallow_any_explicit:
            cli_args.append("--disallow-any-explicit")

        if disallow_any_generics:
            cli_args.append("--disallow-any-generics")

        if disallow_subclassing_any:
            cli_args.append("--disallow-subclassing-any")

        if disallow_untyped_calls:
            cli_args.append("--disallow-untyped-calls")

        if disallow_untyped_defs:
            cli_args.append("--disallow-untyped-defs")

        if disallow_incomplete_defs:
            cli_args.append("--disallow-incomplete-defs")

        if check_untyped_defs:
            cli_args.append("--check-untyped-defs")

        if disallow_untyped_decorators:
            cli_args.append("--disallow-untyped-decorators")

        if implicit_optional:
            cli_args.append("--implicit-optional")

        if no_strict_optional:
            cli_args.append("--no-strict-optional")

        if warn_redundant_casts:
            cli_args.append("--warn-redundant-casts")

        if warn_unused_ignores:
            cli_args.append("--warn-unused-ignores")

        if no_warn_no_return:
            cli_args.append("--no-warn-no-return")

        if warn_return_any:
            cli_args.append("--warn-return-any")

        if warn_unreachable:
            cli_args.append("--warn-unreachable")

        if allow_untyped_globals:
            cli_args.append("--allow-untyped-globals")

        if allow_redefinition:
            cli_args.append("--allow-redefinition")

        if no_implicit_reexport:
            cli_args.append("--no-implicit-reexport")

        if strict_equality:
            cli_args.append("--strict-equality")

        if strict_concatenate:
            cli_args.append("--strict-concatenate")

        if strict:
            cli_args.append("--strict")

        if disable_error_code:
            cli_args.append("--disable-error-code")
            cli_args.append(disable_error_code)

        if enable_error_code:
            cli_args.append("--enable-error-code")
            cli_args.append(enable_error_code)

        if show_error_context:
            cli_args.append("--show-error-context")

        if show_column_numbers:
            cli_args.append("--show-column-numbers")

        if show_error_end:
            cli_args.append("--show-error-end")

        if hide_error_codes:
            cli_args.append("--hide-error-codes")

        if pretty:
            cli_args.append("--pretty")

        if no_color_output:
            cli_args.append("--no-color-output")

        if no_error_summary:
            cli_args.append("--no-error-summary")

        if show_absolute_path:
            cli_args.append("--show-absolute-path")

        if no_incremental:
            cli_args.append("--no-incremental")

        if cache_dir:
            cli_args.append("--cache-dir")
            cli_args.append(cache_dir)

        if sqlite_cache:
            cli_args.append("--sqlite-cache")

        if cache_fine_grained:
            cli_args.append("--cache-fine-grained")

        if skip_version_check:
            cli_args.append("--skip-version-check")

        if skip_cache_mtime_checks:
            cli_args.append("--skip-cache-mtime-checks")

        if pdb:
            cli_args.append("--pdb")

        if show_traceback:
            cli_args.append("--show-traceback")

        if raise_exceptions:
            cli_args.append("--raise-exceptions")

        if custom_typing_module:
            cli_args.append("--custom-typing-module")
            cli_args.append(custom_typing_module)

        if disable_recursive_aliases:
            cli_args.append("--disable-recursive-aliases")

        if custom_typeshed_dir:
            cli_args.append("--custom-typeshed-dir")
            cli_args.append(custom_typeshed_dir)

        if warn_incomplete_stub:
            cli_args.append("--warn-incomplete-stub")

        if shadow_file:
            cli_args.append("--shadow-file")
            cli_args.extend(shadow_file)

        if any_exprs_report:
            cli_args.append("--any-exprs-report")
            cli_args.append(any_exprs_report)

        if cobertura_xml_report:
            cli_args.append("--cobertura-xml-report")
            cli_args.append(cobertura_xml_report)

        if html_report:
            cli_args.append("--html-report")
            cli_args.append(html_report)

        if linecount_report:
            cli_args.append("--linecount-report")
            cli_args.append(linecount_report)

        if linecoverage_report:
            cli_args.append("--linecoverage-report")
            cli_args.append(linecoverage_report)

        if lineprecision_report:
            cli_args.append("--lineprecision-report")
            cli_args.append(lineprecision_report)

        if txt_report:
            cli_args.append("--txt-report")
            cli_args.append(txt_report)

        if xml_report:
            cli_args.append("--xml-report")
            cli_args.append(xml_report)

        if xslt_html_report:
            cli_args.append("--xslt-html-report")
            cli_args.append(xslt_html_report)

        if xslt_txt_report:
            cli_args.append("--xslt-txt-report")
            cli_args.append(xslt_txt_report)

        if junit_xml:
            cli_args.append("--junit-xml")
            cli_args.append(junit_xml)

        if find_occurrences:
            cli_args.append("--find-occurrences")
            cli_args.append(find_occurrences)

        if scripts_are_modules:
            cli_args.append("--scripts-are-modules")

        if install_types:
            cli_args.append("--install-types")

        if non_interactive:
            cli_args.append("--non-interactive")

        if explicit_package_bases:
            cli_args.append("--explicit-package-bases")

        if exclude:
            cli_args.append("--exclude")
            cli_args.append(exclude)

        if module:
            cli_args.append("--module")
            cli_args.append(module)

        if package:
            cli_args.append("--package")
            cli_args.append(package)

        if command:
            cli_args.append("--command")
            cli_args.append(command)

        super().__init__(cli_args)

    def __call__(self) -> None:
        """Run the command."""
        from mypy.main import main as run_mypy  # noqa: PLC0415

        run_mypy(
            args=self.cli_args,
            stdout=LazyStdout(),
            stderr=LazyStderr(),
            clean_exit=True,
        )



# YORE: Bump 2: Remove file.

from __future__ import annotations

from failprint import lazy


@lazy(name="black")
def run(
    *src: str,
    config: str | None = None,
    code: str | None = None,
    line_length: int | None = None,
    target_version: str | None = None,
    check: bool | None = None,
    diff: bool | None = None,
    color: bool | None = None,
    fast: bool | None = None,
    pyi: bool | None = None,
    ipynb: bool | None = None,
    python_cell_magics: str | None = None,
    skip_source_first_line: bool | None = None,
    skip_string_normalization: bool | None = None,
    skip_magic_trailing_comma: bool | None = None,
    experimental_string_processing: bool | None = None,
    preview: bool | None = None,
    quiet: bool | None = None,
    verbose: bool | None = None,
    required_version: str | None = None,
    include: str | None = None,
    exclude: str | None = None,
    extend_exclude: str | None = None,
    force_exclude: str | None = None,
    stdin_filename: str | None = None,
    workers: int | None = None,
) -> None:
    r"""Run `black`.

    Parameters:
        src: Format the directories and file paths.
        config: Read configuration from this file path.
        code: Format the code passed in as a string.
        line_length: How many characters per line to allow [default: 120].
        target_version: Python versions that should be supported by Black's output.
            By default, Black will try to infer this from the project metadata in pyproject.toml.
            If this does not yield conclusive results, Black will use per-file auto-detection.
        check: Don't write the files back, just return the status. Return code 0 means nothing would change.
            Return code 1 means some files would be reformatted. Return code 123 means there was an internal error.
        diff: Don't write the files back, just output a diff for each file on stdout.
        color: Show colored diff. Only applies when `--diff` is given.
        fast: If --fast given, skip temporary sanity checks. [default: --safe]
        pyi: Format all input files like typing stubs regardless of file extension
            (useful when piping source on standard input).
        ipynb: Format all input files like Jupyter Notebooks regardless of file extension
            (useful when piping source on standard input).
        python_cell_magics: When processing Jupyter Notebooks, add the given magic to the list of known python-magics
            (capture, prun, pypy, python, python3, time, timeit). Useful for formatting cells with custom python magics.
        skip_source_first_line: Skip the first line of the source code.
        skip_string_normalization: Don't normalize string quotes or prefixes.
        skip_magic_trailing_comma: Don't use trailing commas as a reason to split lines.
        preview: Enable potentially disruptive style changes that may be added
            to Black's main functionality in the next major release.
        quiet: Don't emit non-error messages to stderr. Errors are still emitted; silence those with 2>/dev/null.
        verbose: Also emit messages to stderr about files that were not changed or were ignored due to exclusion patterns.
        required_version: Require a specific version of Black to be running (useful for unifying results
            across many environments e.g. with a pyproject.toml file).
            It can be either a major version number or an exact version.
        include: A regular expression that matches files and directories that should be included on recursive searches.
            An empty value means all files are included regardless of the name. Use forward slashes for directories
            on all platforms (Windows, too). Exclusions are calculated first, inclusions later [default: (\.pyi?|\.ipynb)$].
        exclude: A regular expression that matches files and directories that should be excluded on recursive searches.
            An empty value means no paths are excluded. Use forward slashes for directories on all platforms (Windows, too).
            Exclusions are calculated first, inclusions later [default: /(\.direnv|\.eggs|\.git|\.hg|\.mypy_cache|\.nox|
            \.tox|\.venv|venv|\.svn|\.ipynb_checkpoints|_build|buck-out|build|dist|__pypackages__)/].
        extend_exclude: Like --exclude, but adds additional files and directories on top of the excluded ones
            (useful if you simply want to add to the default).
        force_exclude: Like --exclude, but files and directories matching this regex will be excluded
            even when they are passed explicitly as arguments.
        stdin_filename: The name of the file when passing it through stdin. Useful to make sure Black will respect
            --force-exclude option on some editors that rely on using stdin.
        workers: Number of parallel workers [default: number CPUs in the system].
    """
    from black import main as black  # noqa: PLC0415

    cli_args = list(src)

    if config:
        cli_args.append("--config")
        cli_args.append(config)

    if code:
        cli_args.append("--code")
        cli_args.append(code)

    if line_length:
        cli_args.append("--line-length")
        cli_args.append(str(line_length))

    if target_version:
        cli_args.append("--target-version")
        cli_args.append(target_version)

    if check:
        cli_args.append("--check")

    if diff:
        cli_args.append("--diff")

    if color is True:
        cli_args.append("--color")
    elif color is False:
        cli_args.append("--no-color")

    if fast:
        cli_args.append("--fast")

    if pyi:
        cli_args.append("--pyi")

    if ipynb:
        cli_args.append("--ipynb")

    if python_cell_magics:
        cli_args.append("--python-cell-magics")
        cli_args.append(python_cell_magics)

    if skip_source_first_line:
        cli_args.append("--skip_source_first_line")

    if skip_string_normalization:
        cli_args.append("--skip_string_normalization")

    if skip_magic_trailing_comma:
        cli_args.append("--skip_magic_trailing_comma")

    if experimental_string_processing:
        cli_args.append("--experimental_string_processing")

    if preview:
        cli_args.append("--preview")

    if quiet:
        cli_args.append("--quiet")

    if verbose:
        cli_args.append("--verbose")

    if required_version:
        cli_args.append("--required-version")
        cli_args.append(required_version)

    if include:
        cli_args.append("--include")
        cli_args.append(include)

    if exclude:
        cli_args.append("--exclude")
        cli_args.append(exclude)

    if extend_exclude:
        cli_args.append("--extend-exclude")
        cli_args.append(extend_exclude)

    if force_exclude:
        cli_args.append("--force-exclude")
        cli_args.append(force_exclude)

    if stdin_filename:
        cli_args.append("--stdin-filename")
        cli_args.append(stdin_filename)

    if workers:
        cli_args.append("--workers")
        cli_args.append(str(workers))

    return black(cli_args, prog_name="black")



from __future__ import annotations

from typing import Any

from duty._internal.tools._base import Tool


class twine(Tool):  # noqa: N801
    """Call [Twine](https://github.com/pypa/twine)."""

    cli_name = "twine"
    """The name of the executable on PATH."""

    @classmethod
    def check(
        cls,
        *dists: str,
        strict: bool = False,
        version: bool = False,
        no_color: bool = False,
    ) -> twine:
        """Checks whether your distribution's long description will render correctly on PyPI.

        Parameters:
            dists: The distribution files to check, usually `dist/*`.
            strict: Fail on warnings.
            version: Show program's version number and exit.
            no_color: Disable colored output.
        """
        cli_args = ["check", *dists]

        if version:
            cli_args.append("--version")

        if no_color:
            cli_args.append("--no-color")

        if strict is True:
            cli_args.append("--strict")

        return cls(cli_args)

    @classmethod
    def register(
        cls,
        package: str,
        *,
        repository: str = "pypi",
        repository_url: str | None = None,
        attestations: bool = False,
        sign: bool = False,
        sign_with: str | None = None,
        identity: str | None = None,
        username: str | None = None,
        password: str | None = None,
        non_interactive: bool = False,
        comment: str | None = None,
        config_file: str | None = None,
        skip_existing: bool = False,
        cert: str | None = None,
        client_cert: str | None = None,
        verbose: bool = False,
        disable_progress_bar: bool = False,
        version: bool = False,
        no_color: bool = False,
    ) -> twine:
        """Pre-register a name with a repository before uploading a distribution.

        Pre-registration is not supported on PyPI, so the register command
        is only necessary if you are using a different repository that requires it.

        Parameters:
            package: File from which we read the package metadata.
            repository: The repository (package index) to upload the package to.
                Should be a section in the config file (default: `pypi`).
                Can also be set via `TWINE_REPOSITORY` environment variable.
            repository_url: The repository (package index) URL to upload the package to. This overrides `--repository`.
                Can also be set via `TWINE_REPOSITORY_URL` environment variable.
            attestations: Upload each file's associated attestations.
            sign: Sign files to upload using GPG.
            sign_with: GPG program used to sign uploads (default: `gpg`).
            identity: GPG identity used to sign files.
            username: The username to authenticate to the repository (package index) as.
                Can also be set via `TWINE_USERNAME` environment variable.
            password: The password to authenticate to the repository (package index) with.
                Can also be set via `TWINE_PASSWORD` environment variable.
            non_interactive: Do not interactively prompt for username/password if the required credentials are missing.
                Can also be set via `TWINE_NON_INTERACTIVE` environment variable.
            comment: The comment to include with the distribution file.
            config_file: The `.pypirc` config file to use.
            skip_existing: Continue uploading files if one already exists.
                Only valid when uploading to PyPI. Other implementations may not support this.
            cert: Path to alternate CA bundle (can also be set via `TWINE_CERT` environment variable).
            client_cert: Path to SSL client certificate, a single file containing the private key and the certificate in PEM format.
            verbose: Show verbose output.
            disable_progress_bar: Disable the progress bar.
            version: Show program's version number and exit.
            no_color: Disable colored output.
        """
        cli_args = ["register", package]

        if version:
            cli_args.append("--version")

        if no_color:
            cli_args.append("--no-color")

        if repository:
            cli_args.append("--repository")
            cli_args.append(repository)

        if repository_url:
            cli_args.append("--repository-url")
            cli_args.append(repository_url)

        if attestations:
            cli_args.append("--attestations")

        if sign:
            cli_args.append("--sign")

        if sign_with:
            cli_args.append("--sign-with")
            cli_args.append(sign_with)

        if identity:
            cli_args.append("--identity")
            cli_args.append(identity)

        if username:
            cli_args.append("--username")
            cli_args.append(username)

        if password:
            cli_args.append("--password")
            cli_args.append(password)

        if non_interactive:
            cli_args.append("--non-interactive")

        if comment:
            cli_args.append("--repository")

        if config_file:
            cli_args.append("--config-file")
            cli_args.append(config_file)

        if skip_existing:
            cli_args.append("--skip-existing")

        if cert:
            cli_args.append("--cert")
            cli_args.append(cert)

        if client_cert:
            cli_args.append("--client-cert")
            cli_args.append(client_cert)

        if verbose:
            cli_args.append("--verbose")

        if disable_progress_bar:
            cli_args.append("--disable-progress-bar")

        return cls(cli_args)

    @classmethod
    def upload(
        cls,
        *dists: str,
        repository: str = "pypi",
        repository_url: str | None = None,
        attestations: bool = False,
        sign: bool = False,
        sign_with: str | None = None,
        identity: str | None = None,
        username: str | None = None,
        password: str | None = None,
        non_interactive: bool = False,
        comment: str | None = None,
        config_file: str | None = None,
        skip_existing: bool = False,
        cert: str | None = None,
        client_cert: str | None = None,
        verbose: bool = False,
        disable_progress_bar: bool = False,
        version: bool = False,
        no_color: bool = False,
    ) -> twine:
        """Uploads one or more distributions to a repository.

        Parameters:
            dists: The distribution files to check, usually `dist/*`.
            repository: The repository (package index) to upload the package to.
                Should be a section in the config file (default: `pypi`).
                Can also be set via `TWINE_REPOSITORY` environment variable.
            repository_url: The repository (package index) URL to upload the package to. This overrides `--repository`.
                Can also be set via `TWINE_REPOSITORY_URL` environment variable.
            attestations: Upload each file's associated attestations.
            sign: Sign files to upload using GPG.
            sign_with: GPG program used to sign uploads (default: `gpg`).
            identity: GPG identity used to sign files.
            username: The username to authenticate to the repository (package index) as.
                Can also be set via `TWINE_USERNAME` environment variable.
            password: The password to authenticate to the repository (package index) with.
                Can also be set via `TWINE_PASSWORD` environment variable.
            non_interactive: Do not interactively prompt for username/password if the required credentials are missing.
                Can also be set via `TWINE_NON_INTERACTIVE` environment variable.
            comment: The comment to include with the distribution file.
            config_file: The `.pypirc` config file to use.
            skip_existing: Continue uploading files if one already exists.
                Only valid when uploading to PyPI. Other implementations may not support this.
            cert: Path to alternate CA bundle (can also be set via `TWINE_CERT` environment variable).
            client_cert: Path to SSL client certificate, a single file containing the private key and the certificate in PEM format.
            verbose: Show verbose output.
            disable_progress_bar: Disable the progress bar.
            version: Show program's version number and exit.
            no_color: Disable colored output.
        """
        cli_args = ["upload", *dists]

        if version:
            cli_args.append("--version")

        if no_color:
            cli_args.append("--no-color")

        if repository:
            cli_args.append("--repository")
            cli_args.append(repository)

        if repository_url:
            cli_args.append("--repository-url")
            cli_args.append(repository_url)

        if attestations:
            cli_args.append("--attestations")

        if sign:
            cli_args.append("--sign")

        if sign_with:
            cli_args.append("--sign-with")
            cli_args.append(sign_with)

        if identity:
            cli_args.append("--identity")
            cli_args.append(identity)

        if username:
            cli_args.append("--username")
            cli_args.append(username)

        if password:
            cli_args.append("--password")
            cli_args.append(password)

        if non_interactive:
            cli_args.append("--non-interactive")

        if comment:
            cli_args.append("--repository")

        if config_file:
            cli_args.append("--config-file")
            cli_args.append(config_file)

        if skip_existing:
            cli_args.append("--skip-existing")

        if cert:
            cli_args.append("--cert")
            cli_args.append(cert)

        if client_cert:
            cli_args.append("--client-cert")
            cli_args.append(client_cert)

        if verbose:
            cli_args.append("--verbose")

        if disable_progress_bar:
            cli_args.append("--disable-progress-bar")

        return cls(cli_args)

    def __call__(self) -> Any:
        """Run the command.

        Returns:
            The return value of the corresponding Twine command / entrypoint.
        """
        from twine.cli import dispatch as run_twine  # noqa: PLC0415

        return run_twine(self.cli_args)



# YORE: Bump 2: Remove file.

from __future__ import annotations

from typing import Literal

from failprint import lazy


def _run(args: list[str]) -> None:
    from coverage.cmdline import main as coverage  # noqa: PLC0415

    coverage(args)


@lazy(name="coverage.annotate")
def annotate(
    *,
    rcfile: str | None = None,
    directory: str | None = None,
    data_file: str | None = None,
    ignore_errors: bool | None = None,
    include: list[str] | None = None,
    omit: list[str] | None = None,
    debug_opts: list[str] | None = None,
) -> None:
    """Annotate source files with execution information.

    Make annotated copies of the given files, marking statements that are executed
    with `>` and statements that are missed with `!`.

    Parameters:
        rcfile: Specify configuration file. By default `.coveragerc`, `setup.cfg`, `tox.ini`,
            and `pyproject.toml` are tried [env: `COVERAGE_RCFILE`].
        directory: Write the output files to this directory.
        data_file: Read coverage data for report generation from this file.
            Defaults to `.coverage` [env: `COVERAGE_FILE`].
        ignore_errors: Ignore errors while reading source files.
        include: Include only files whose paths match one of these patterns. Accepts shell-style wildcards, which must be quoted.
        omit: Omit files whose paths match one of these patterns. Accepts shell-style wildcards, which must be quoted.
        debug_opts: Debug options, separated by commas [env: `COVERAGE_DEBUG`].
    """
    cli_args = ["annotate"]

    if directory:
        cli_args.append("--directory")
        cli_args.append(directory)

    if data_file:
        cli_args.append("--data-file")
        cli_args.append(data_file)

    if ignore_errors:
        cli_args.append("--ignore-errors")

    if include:
        cli_args.append("--include")
        cli_args.append(",".join(include))

    if omit:
        cli_args.append("--omit")
        cli_args.append(",".join(omit))

    if debug_opts:
        cli_args.append("--debug")
        cli_args.append(",".join(debug_opts))

    if rcfile:
        cli_args.append("--rcfile")
        cli_args.append(rcfile)

    _run(cli_args)


@lazy(name="coverage.combine")
def combine(
    *paths: str,
    rcfile: str | None = None,
    append: bool | None = None,
    data_file: str | None = None,
    keep: bool | None = None,
    quiet: bool | None = None,
    debug_opts: list[str] | None = None,
) -> None:
    """Combine a number of data files.

    Combine data from multiple coverage files. The combined results are written to
    a single file representing the union of the data. The positional arguments are
    data files or directories containing data files. If no paths are provided,
    data files in the default data file's directory are combined.

    Parameters:
        paths: Paths to combine.
        rcfile: Specify configuration file. By default `.coveragerc`, `setup.cfg`, `tox.ini`,
            and `pyproject.toml` are tried [env: `COVERAGE_RCFILE`].
        append: Append coverage data to .coverage, otherwise it starts clean each time.
        data_file: Read coverage data for report generation from this file.
            Defaults to `.coverage` [env: `COVERAGE_FILE`].
        keep: Keep original coverage files, otherwise they are deleted.
        quiet: Don't print messages about what is happening.
        debug_opts: Debug options, separated by commas [env: `COVERAGE_DEBUG`].
    """
    cli_args = ["combine", *paths]

    if append:
        cli_args.append("--append")

    if data_file:
        cli_args.append("--data-file")
        cli_args.append(data_file)

    if keep:
        cli_args.append("--keep")

    if quiet:
        cli_args.append("--quiet")

    if debug_opts:
        cli_args.append("--debug")
        cli_args.append(",".join(debug_opts))

    if rcfile:
        cli_args.append("--rcfile")
        cli_args.append(rcfile)

    _run(cli_args)


@lazy(name="coverage.debug")
def debug(
    topic: Literal["data", "sys", "config", "premain", "pybehave"],
    *,
    rcfile: str | None = None,
    debug_opts: list[str] | None = None,
) -> None:
    """Display information about the internals of coverage.py.

    Display information about the internals of coverage.py, for diagnosing
    problems. Topics are: `data` to show a summary of the collected data; `sys` to
    show installation information; `config` to show the configuration; `premain`
    to show what is calling coverage; `pybehave` to show internal flags describing
    Python behavior.

    Parameters:
        topic: Topic to display.
        rcfile: Specify configuration file. By default `.coveragerc`, `setup.cfg`, `tox.ini`,
            and `pyproject.toml` are tried [env: `COVERAGE_RCFILE`].
        debug_opts: Debug options, separated by commas [env: `COVERAGE_DEBUG`].
    """
    cli_args: list[str] = ["debug", topic]

    if debug_opts:
        cli_args.append("--debug")
        cli_args.append(",".join(debug_opts))

    if rcfile:
        cli_args.append("--rcfile")
        cli_args.append(rcfile)

    _run(cli_args)


@lazy(name="coverage.erase")
def erase(
    *,
    rcfile: str | None = None,
    data_file: str | None = None,
    debug_opts: list[str] | None = None,
) -> None:
    """Erase previously collected coverage data.

    Parameters:
        rcfile: Specify configuration file. By default `.coveragerc`, `setup.cfg`, `tox.ini`,
            and `pyproject.toml` are tried [env: `COVERAGE_RCFILE`].
        data_file: Read coverage data for report generation from this file.
            Defaults to `.coverage` [env: `COVERAGE_FILE`].
        debug_opts: Debug options, separated by commas [env: `COVERAGE_DEBUG`].
    """
    cli_args = ["erase"]

    if data_file:
        cli_args.append("--data-file")
        cli_args.append(data_file)

    if debug_opts:
        cli_args.append("--debug")
        cli_args.append(",".join(debug_opts))

    if rcfile:
        cli_args.append("--rcfile")
        cli_args.append(rcfile)

    _run(cli_args)


@lazy(name="coverage.html")
def html(
    *,
    rcfile: str | None = None,
    contexts: list[str] | None = None,
    directory: str | None = None,
    data_file: str | None = None,
    fail_under: int | None = None,
    ignore_errors: bool | None = None,
    include: list[str] | None = None,
    omit: list[str] | None = None,
    precision: int | None = None,
    quiet: bool | None = None,
    show_contexts: bool | None = None,
    skip_covered: bool | None = None,
    skip_empty: bool | None = None,
    title: str | None = None,
    debug_opts: list[str] | None = None,
) -> None:
    """Create an HTML report.

    Create an HTML report of the coverage of the files.  Each file gets its own
    page, with the source decorated to show executed, excluded, and missed lines.

    Parameters:
        rcfile: Specify configuration file. By default `.coveragerc`, `setup.cfg`, `tox.ini`,
            and `pyproject.toml` are tried [env: `COVERAGE_RCFILE`].
        contexts: Only display data from lines covered in the given contexts.
            Accepts Python regexes, which must be quoted.
        directory: Write the output files to this directory.
        data_file: Read coverage data for report generation from this file.
            Defaults to `.coverage` [env: `COVERAGE_FILE`].
        fail_under: Exit with a status of 2 if the total coverage is less than the given number.
        ignore_errors: Ignore errors while reading source files.
        include: Include only files whose paths match one of these patterns. Accepts shell-style wildcards, which must be quoted.
        omit: Omit files whose paths match one of these patterns. Accepts shell-style wildcards, which must be quoted.
        precision: Number of digits after the decimal point to display for reported coverage percentages.
        quiet: Don't print messages about what is happening.
        show_contexts: Show contexts for covered lines.
        skip_covered: Skip files with 100% coverage.
        skip_empty: Skip files with no code.
        title: A text string to use as the title on the HTML.
        debug_opts: Debug options, separated by commas [env: `COVERAGE_DEBUG`].
    """
    cli_args = ["html"]

    if contexts:
        cli_args.append("--contexts")
        cli_args.append(",".join(contexts))

    if directory:
        cli_args.append("--directory")
        cli_args.append(directory)

    if data_file:
        cli_args.append("--data-file")
        cli_args.append(data_file)

    if fail_under is not None:
        cli_args.append("--fail-under")
        cli_args.append(str(fail_under))

    if ignore_errors:
        cli_args.append("--ignore-errors")

    if include:
        cli_args.append("--include")
        cli_args.append(",".join(include))

    if omit:
        cli_args.append("--omit")
        cli_args.append(",".join(omit))

    if precision is not None:
        cli_args.append("--precision")
        cli_args.append(str(precision))

    if quiet:
        cli_args.append("--quiet")

    if show_contexts:
        cli_args.append("--show-contexts")

    if skip_covered is True:
        cli_args.append("--skip-covered")
    elif skip_covered is False:
        cli_args.append("--no-skip-covered")

    if skip_empty:
        cli_args.append("--skip-empty")

    if title:
        cli_args.append("--title")
        cli_args.append(title)

    if debug_opts:
        cli_args.append("--debug")
        cli_args.append(",".join(debug_opts))

    if rcfile:
        cli_args.append("--rcfile")
        cli_args.append(rcfile)

    _run(cli_args)


@lazy(name="coverage.json")
def json(
    *,
    rcfile: str | None = None,
    contexts: list[str] | None = None,
    data_file: str | None = None,
    fail_under: int | None = None,
    ignore_errors: bool | None = None,
    include: list[str] | None = None,
    omit: list[str] | None = None,
    output: str | None = None,
    pretty_print: bool | None = None,
    quiet: bool | None = None,
    show_contexts: bool | None = None,
    debug_opts: list[str] | None = None,
) -> None:
    """Create a JSON report of coverage results.

    Parameters:
        rcfile: Specify configuration file. By default `.coveragerc`, `setup.cfg`, `tox.ini`,
            and `pyproject.toml` are tried [env: `COVERAGE_RCFILE`].
        contexts: Only display data from lines covered in the given contexts.
            Accepts Python regexes, which must be quoted.
        data_file: Read coverage data for report generation from this file.
            Defaults to `.coverage` [env: `COVERAGE_FILE`].
        fail_under: Exit with a status of 2 if the total coverage is less than the given number.
        ignore_errors: Ignore errors while reading source files.
        include: Include only files whose paths match one of these patterns. Accepts shell-style wildcards, which must be quoted.
        omit: Omit files whose paths match one of these patterns. Accepts shell-style wildcards, which must be quoted.
        output: Write the JSON report to this file. Defaults to `coverage.json`.
        pretty_print: Format the JSON for human readers.
        quiet: Don't print messages about what is happening.
        show_contexts: Show contexts for covered lines.
        debug_opts: Debug options, separated by commas [env: `COVERAGE_DEBUG`].
    """
    cli_args = ["json"]

    if contexts:
        cli_args.append("--contexts")
        cli_args.append(",".join(contexts))

    if data_file:
        cli_args.append("--data-file")
        cli_args.append(data_file)

    if fail_under is not None:
        cli_args.append("--fail-under")
        cli_args.append(str(fail_under))

    if ignore_errors:
        cli_args.append("--ignore-errors")

    if include:
        cli_args.append("--include")
        cli_args.append(",".join(include))

    if omit:
        cli_args.append("--omit")
        cli_args.append(",".join(omit))

    if output:
        cli_args.append("-o")
        cli_args.append(output)

    if pretty_print:
        cli_args.append("--pretty-print")

    if quiet:
        cli_args.append("--quiet")

    if show_contexts:
        cli_args.append("--show-contexts")

    if debug_opts:
        cli_args.append("--debug")
        cli_args.append(",".join(debug_opts))

    if rcfile:
        cli_args.append("--rcfile")
        cli_args.append(rcfile)

    _run(cli_args)


@lazy(name="coverage.lcov")
def lcov(
    *,
    rcfile: str | None = None,
    data_file: str | None = None,
    fail_under: int | None = None,
    ignore_errors: bool | None = None,
    include: list[str] | None = None,
    omit: list[str] | None = None,
    output: str | None = None,
    quiet: bool | None = None,
    debug_opts: list[str] | None = None,
) -> None:
    """Create an LCOV report of coverage results.

    Parameters:
        rcfile: Specify configuration file. By default `.coveragerc`, `setup.cfg`, `tox.ini`,
            and `pyproject.toml` are tried [env: `COVERAGE_RCFILE`].
        data_file: Read coverage data for report generation from this file.
            Defaults to `.coverage` [env: `COVERAGE_FILE`].
        fail_under: Exit with a status of 2 if the total coverage is less than the given number.
        ignore_errors: Ignore errors while reading source files.
        include: Include only files whose paths match one of these patterns. Accepts shell-style wildcards, which must be quoted.
        omit: Omit files whose paths match one of these patterns. Accepts shell-style wildcards, which must be quoted.
        output: Write the JSON report to this file. Defaults to `coverage.json`.
        quiet: Don't print messages about what is happening.
        debug_opts: Debug options, separated by commas [env: `COVERAGE_DEBUG`].
    """
    cli_args = ["lcov"]

    if data_file:
        cli_args.append("--data-file")
        cli_args.append(data_file)

    if fail_under is not None:
        cli_args.append("--fail-under")
        cli_args.append(str(fail_under))

    if ignore_errors:
        cli_args.append("--ignore-errors")

    if include:
        cli_args.append("--include")
        cli_args.append(",".join(include))

    if omit:
        cli_args.append("--omit")
        cli_args.append(",".join(omit))

    if output:
        cli_args.append("-o")
        cli_args.append(output)

    if quiet:
        cli_args.append("--quiet")

    if debug_opts:
        cli_args.append("--debug")
        cli_args.append(",".join(debug_opts))

    if rcfile:
        cli_args.append("--rcfile")
        cli_args.append(rcfile)

    _run(cli_args)


@lazy(name="coverage.report")
def report(
    *,
    rcfile: str | None = None,
    contexts: list[str] | None = None,
    data_file: str | None = None,
    fail_under: int | None = None,
    output_format: Literal["text", "markdown", "total"] | None = None,
    ignore_errors: bool | None = None,
    include: list[str] | None = None,
    omit: list[str] | None = None,
    precision: int | None = None,
    sort: Literal["name", "stmts", "miss", "branch", "brpart", "cover"] | None = None,
    show_missing: bool | None = None,
    skip_covered: bool | None = None,
    skip_empty: bool | None = None,
    debug_opts: list[str] | None = None,
) -> None:
    """Report coverage statistics on modules.

    Parameters:
        rcfile: Specify configuration file. By default `.coveragerc`, `setup.cfg`, `tox.ini`,
            and `pyproject.toml` are tried [env: `COVERAGE_RCFILE`].
        contexts: Only display data from lines covered in the given contexts.
        data_file: Read coverage data for report generation from this file.
            Defaults to `.coverage` [env: `COVERAGE_FILE`].
        fail_under: Exit with a status of 2 if the total coverage is less than the given number.
        output_format: Output format, either text (default), markdown, or total.
        ignore_errors: Ignore errors while reading source files.
        include: Include only files whose paths match one of these patterns. Accepts shell-style wildcards, which must be quoted.
        omit: Omit files whose paths match one of these patterns. Accepts shell-style wildcards, which must be quoted.
        precision: Number of digits after the decimal point to display for reported coverage percentages.
        sort: Sort the report by the named column: name, stmts, miss, branch, brpart, or cover. Default is name.
        show_missing: Show line numbers of statements in each module that weren't executed.
        skip_covered: Skip files with 100% coverage.
        skip_empty: Skip files with no code.
        debug_opts: Debug options, separated by commas [env: `COVERAGE_DEBUG`].
    """
    cli_args = ["report"]

    if contexts:
        cli_args.append("--contexts")
        cli_args.append(",".join(contexts))

    if data_file:
        cli_args.append("--data-file")
        cli_args.append(data_file)

    if fail_under is not None:
        cli_args.append("--fail-under")
        cli_args.append(str(fail_under))

    if output_format:
        cli_args.append("--format")
        cli_args.append(output_format)

    if ignore_errors:
        cli_args.append("--ignore-errors")

    if include:
        cli_args.append("--include")
        cli_args.append(",".join(include))

    if omit:
        cli_args.append("--omit")
        cli_args.append(",".join(omit))

    if precision is not None:
        cli_args.append("--precision")
        cli_args.append(str(precision))

    if sort:
        cli_args.append("--sort")
        cli_args.append(sort)

    if show_missing:
        cli_args.append("--show-missing")

    if skip_covered is True:
        cli_args.append("--skip-covered")
    elif skip_covered is False:
        cli_args.append("--no-skip-covered")

    if skip_empty:
        cli_args.append("--skip-empty")

    if debug_opts:
        cli_args.append("--debug")
        cli_args.append(",".join(debug_opts))

    if rcfile:
        cli_args.append("--rcfile")
        cli_args.append(rcfile)

    _run(cli_args)


@lazy(name="coverage.run")
def run(
    pyfile: str,
    *,
    rcfile: str | None = None,
    append: bool | None = None,
    branch: bool | None = None,
    concurrency: list[str] | None = None,
    context: str | None = None,
    data_file: str | None = None,
    include: list[str] | None = None,
    omit: list[str] | None = None,
    module: bool | None = None,
    pylib: bool | None = None,
    parallel_mode: bool | None = None,
    source: list[str] | None = None,
    timid: bool | None = None,
    debug_opts: list[str] | None = None,
) -> None:
    """Run a Python program and measure code execution.

    Parameters:
        pyfile: Python script or module to run.
        rcfile: Specify configuration file. By default `.coveragerc`, `setup.cfg`, `tox.ini`,
            and `pyproject.toml` are tried [env: `COVERAGE_RCFILE`].
        append: Append coverage data to .coverage, otherwise it starts clean each time.
        branch: Measure branch coverage in addition to statement coverage.
        concurrency: Properly measure code using a concurrency library. Valid values are:
            eventlet, gevent, greenlet, multiprocessing, thread, or a comma-list of them.
        context: The context label to record for this coverage run.
        data_file: Read coverage data for report generation from this file.
            Defaults to `.coverage` [env: `COVERAGE_FILE`].
        include: Include only files whose paths match one of these patterns. Accepts shell-style wildcards, which must be quoted.
        omit: Omit files whose paths match one of these patterns. Accepts shell-style wildcards, which must be quoted.
        module: The given file is an importable Python module, not a script path, to be run as `python -m` would run it.
        pylib: Measure coverage even inside the Python installed library, which isn't done by default.
        parallel_mode: Append the machine name, process id and random number to the data file name
            to simplify collecting data from many processes.
        source: A list of directories or importable names of code to measure.
        timid: Use a simpler but slower trace method. Try this if you get seemingly impossible results!
        debug_opts: Debug options, separated by commas [env: `COVERAGE_DEBUG`].
    """
    cli_args = ["run", pyfile]

    if append:
        cli_args.append("--append")

    if branch:
        cli_args.append("--branch")

    if concurrency:
        cli_args.append("--concurrency")
        cli_args.append(",".join(concurrency))

    if context:
        cli_args.append("--context")
        cli_args.append(context)

    if data_file:
        cli_args.append("--data-file")
        cli_args.append(data_file)

    if include:
        cli_args.append("--include")
        cli_args.append(",".join(include))

    if omit:
        cli_args.append("--omit")
        cli_args.append(",".join(omit))

    if module:
        cli_args.append("--module")

    if pylib:
        cli_args.append("--pylib")

    if parallel_mode:
        cli_args.append("--parallel-mode")

    if source:
        cli_args.append("--source")
        cli_args.append(",".join(source))

    if timid:
        cli_args.append("--timid")

    if debug_opts:
        cli_args.append("--debug")
        cli_args.append(",".join(debug_opts))

    if rcfile:
        cli_args.append("--rcfile")
        cli_args.append(rcfile)

    _run(cli_args)


@lazy(name="coverage.xml")
def xml(
    *,
    rcfile: str | None = None,
    data_file: str | None = None,
    fail_under: int | None = None,
    ignore_errors: bool | None = None,
    include: list[str] | None = None,
    omit: list[str] | None = None,
    output: str | None = None,
    quiet: bool | None = None,
    skip_empty: bool | None = None,
    debug_opts: list[str] | None = None,
) -> None:
    """Create an XML report of coverage results.

    Parameters:
        rcfile: Specify configuration file. By default `.coveragerc`, `setup.cfg`, `tox.ini`,
            and `pyproject.toml` are tried [env: `COVERAGE_RCFILE`].
        data_file: Read coverage data for report generation from this file.
            Defaults to `.coverage` [env: `COVERAGE_FILE`].
        fail_under: Exit with a status of 2 if the total coverage is less than the given number.
        ignore_errors: Ignore errors while reading source files.
        include: Include only files whose paths match one of these patterns. Accepts shell-style wildcards, which must be quoted.
        omit: Omit files whose paths match one of these patterns. Accepts shell-style wildcards, which must be quoted.
        output: Write the JSON report to this file. Defaults to `coverage.json`.
        quiet: Don't print messages about what is happening.
        skip_empty: Skip files with no code.
        debug_opts: Debug options, separated by commas [env: `COVERAGE_DEBUG`].
    """
    cli_args = ["xml"]

    if data_file:
        cli_args.append("--data-file")
        cli_args.append(data_file)

    if fail_under is not None:
        cli_args.append("--fail-under")
        cli_args.append(str(fail_under))

    if ignore_errors:
        cli_args.append("--ignore-errors")

    if include:
        cli_args.append("--include")
        cli_args.append(",".join(include))

    if omit:
        cli_args.append("--omit")
        cli_args.append(",".join(omit))

    if output:
        cli_args.append("-o")
        cli_args.append(output)

    if quiet:
        cli_args.append("--quiet")

    if skip_empty:
        cli_args.append("--skip-empty")

    if debug_opts:
        cli_args.append("--debug")
        cli_args.append(",".join(debug_opts))

    if rcfile:
        cli_args.append("--rcfile")
        cli_args.append(rcfile)

    _run(cli_args)



from __future__ import annotations

from duty._internal.tools._base import Tool


class yore(Tool):  # noqa: N801
    """Call [Yore](https://github.com/pawamoy/yore)."""

    cli_name = "yore"
    """The name of the executable on PATH."""

    @classmethod
    def check(
        cls,
        *paths: str,
        bump: str | None = None,
        eol_within: str | None = None,
        bol_within: str | None = None,
    ) -> yore:
        """Check Yore comments against Python EOL dates or the provided next version of your project.

        Parameters:
            paths: Path to files or directories to check.
            bump: The next version of your project.
            eol_within: The time delta to start checking before the End of Life of a Python version.
                It is provided in a human-readable format, like `2 weeks` or `1 month`.
                Spaces are optional, and the unit can be shortened to a single letter:
                `d` for days, `w` for weeks, `m` for months, and `y` for years.
            bol_within: The time delta to start checking before the Beginning of Life of a Python version.
                It is provided in a human-readable format, like `2 weeks` or `1 month`.
                Spaces are optional, and the unit can be shortened to a single letter:
                `d` for days, `w` for weeks, `m` for months, and `y` for years.
        """
        cli_args = ["check", *paths]

        if bump:
            cli_args.append("--bump")
            cli_args.append(bump)

        if eol_within:
            cli_args.append("--eol-within")
            cli_args.append(eol_within)

        if bol_within:
            cli_args.append("--bol-within")
            cli_args.append(bol_within)

        return cls(cli_args)

    @classmethod
    def fix(
        cls,
        *paths: str,
        bump: str | None = None,
        eol_within: str | None = None,
        bol_within: str | None = None,
    ) -> yore:
        """Fix your code by transforming it according to the Yore comments.

        Parameters:
            paths: Path to files or directories to fix.
            bump: The next version of your project.
            eol_within: The time delta to start fixing before the End of Life of a Python version.
                It is provided in a human-readable format, like `2 weeks` or `1 month`.
                Spaces are optional, and the unit can be shortened to a single letter:
                `d` for days, `w` for weeks, `m` for months, and `y` for years.
            bol_within: The time delta to start fixing before the Beginning of Life of a Python version.
                It is provided in a human-readable format, like `2 weeks` or `1 month`.
                Spaces are optional, and the unit can be shortened to a single letter:
                `d` for days, `w` for weeks, `m` for months, and `y` for years.
        """
        cli_args = ["fix", *paths]

        if bump:
            cli_args.append("--bump")
            cli_args.append(bump)

        if eol_within:
            cli_args.append("--eol-within")
            cli_args.append(eol_within)

        if bol_within:
            cli_args.append("--bol-within")
            cli_args.append(bol_within)

        return cls(cli_args)

    def __call__(self) -> int:
        """Run the command.

        Returns:
            The exit code of the command.
        """
        from yore import main as run_yore  # noqa: PLC0415

        return run_yore(self.cli_args)



"""Module containing callables for many tools.

These callables are **deprecated** in favor of our new [tools][duty.tools].
"""

# YORE: Bump 2: Remove file.

from __future__ import annotations

import warnings

from failprint import lazy  # noqa: F401

from duty._internal.callables import (
    autoflake,  # noqa: F401
    black,  # noqa: F401
    blacken_docs,  # noqa: F401
    build,  # noqa: F401
    coverage,  # noqa: F401
    flake8,  # noqa: F401
    git_changelog,  # noqa: F401
    griffe,  # noqa: F401
    interrogate,  # noqa: F401
    isort,  # noqa: F401
    mkdocs,  # noqa: F401
    mypy,  # noqa: F401
    pytest,  # noqa: F401
    ruff,  # noqa: F401
    safety,  # noqa: F401
    ssort,  # noqa: F401
    twine,  # noqa: F401
)

warnings.warn(
    "Callables are deprecated in favor of our new `duty.tools`. "
    "They are easier to use and provide more functionality "
    "like automatically computing `command` values in `ctx.run()` calls. "
    "Old callables will be removed in a future version.",
    DeprecationWarning,
    stacklevel=1,
)



# YORE: Bump 2: Remove file.

from __future__ import annotations

from typing import Literal

from failprint import lazy

from duty._internal.callables._io import _LazyStderr, _LazyStdout


@lazy(name="mypy")
def run(
    *paths: str,
    config_file: str | None = None,
    enable_incomplete_feature: bool | None = None,
    verbose: bool | None = None,
    warn_unused_configs: bool | None = None,
    no_namespace_packages: bool | None = None,
    ignore_missing_imports: bool | None = None,
    follow_imports: Literal["normal", "silent", "skip", "error"] | None = None,
    python_executable: str | None = None,
    no_site_packages: bool | None = None,
    no_silence_site_packages: bool | None = None,
    python_version: str | None = None,
    py2: bool | None = None,
    platform: str | None = None,
    always_true: list[str] | None = None,
    always_false: list[str] | None = None,
    disallow_any_unimported: bool | None = None,
    disallow_any_expr: bool | None = None,
    disallow_any_decorated: bool | None = None,
    disallow_any_explicit: bool | None = None,
    disallow_any_generics: bool | None = None,
    disallow_subclassing_any: bool | None = None,
    disallow_untyped_calls: bool | None = None,
    disallow_untyped_defs: bool | None = None,
    disallow_incomplete_defs: bool | None = None,
    check_untyped_defs: bool | None = None,
    disallow_untyped_decorators: bool | None = None,
    implicit_optional: bool | None = None,
    no_strict_optional: bool | None = None,
    warn_redundant_casts: bool | None = None,
    warn_unused_ignores: bool | None = None,
    no_warn_no_return: bool | None = None,
    warn_return_any: bool | None = None,
    warn_unreachable: bool | None = None,
    allow_untyped_globals: bool | None = None,
    allow_redefinition: bool | None = None,
    no_implicit_reexport: bool | None = None,
    strict_equality: bool | None = None,
    strict_concatenate: bool | None = None,
    strict: bool | None = None,
    disable_error_code: str | None = None,
    enable_error_code: str | None = None,
    show_error_context: bool | None = None,
    show_column_numbers: bool | None = None,
    show_error_end: bool | None = None,
    hide_error_codes: bool | None = None,
    pretty: bool | None = None,
    no_color_output: bool | None = None,
    no_error_summary: bool | None = None,
    show_absolute_path: bool | None = None,
    no_incremental: bool | None = None,
    cache_dir: str | None = None,
    sqlite_cache: bool | None = None,
    cache_fine_grained: bool | None = None,
    skip_version_check: bool | None = None,
    skip_cache_mtime_checks: bool | None = None,
    pdb: bool | None = None,
    show_traceback: bool | None = None,
    raise_exceptions: bool | None = None,
    custom_typing_module: str | None = None,
    disable_recursive_aliases: bool | None = None,
    custom_typeshed_dir: str | None = None,
    warn_incomplete_stub: bool | None = None,
    shadow_file: tuple[str, str] | None = None,
    any_exprs_report: str | None = None,
    cobertura_xml_report: str | None = None,
    html_report: str | None = None,
    linecount_report: str | None = None,
    linecoverage_report: str | None = None,
    lineprecision_report: str | None = None,
    txt_report: str | None = None,
    xml_report: str | None = None,
    xslt_html_report: str | None = None,
    xslt_txt_report: str | None = None,
    junit_xml: str | None = None,
    find_occurrences: str | None = None,
    scripts_are_modules: bool | None = None,
    install_types: bool | None = None,
    non_interactive: bool | None = None,
    explicit_package_bases: bool | None = None,
    exclude: str | None = None,
    module: str | None = None,
    package: str | None = None,
    command: str | None = None,
) -> None:
    """Run mypy.

    Parameters:
        *paths: Path to scan.
        config_file: Configuration file, must have a [mypy] section (defaults to mypy.ini, .mypy.ini,
        enable_incomplete_feature: Enable support of incomplete/experimental features for early preview.
        verbose: More verbose messages.
            pyproject.toml, setup.cfg, /home/pawamoy/.config/mypy/config, ~/.config/mypy/config, ~/.mypy.ini).
        warn_unused_configs: Warn about unused '[mypy-<pattern>]' or '[[tool.mypy.overrides]]' config sections
            (inverse: --no-warn-unused-configs).
        no_namespace_packages: Support namespace packages (PEP 420, __init__.py-less) (inverse: --namespace-packages).
        ignore_missing_imports: Silently ignore imports of missing modules.
        follow_imports: How to treat imports (default normal).
        python_executable: Python executable used for finding PEP 561 compliant installed packages and stubs.
        no_site_packages: Do not search for installed PEP 561 compliant packages.
        no_silence_site_packages: Do not silence errors in PEP 561 compliant installed packages.
        python_version: Type check code assuming it will be running on Python x.y.
        py2: Use Python 2 mode (same as --python-version 2.7).
        platform: Type check special-cased code for the given OS platform (defaults to sys.platform).
        always_true: Additional variable to be considered True (may be repeated).
        always_false: Additional variable to be considered False (may be repeated).
        disallow_any_unimported: Disallow Any types resulting from unfollowed imports.
        disallow_any_expr: Disallow all expressions that have type Any.
        disallow_any_decorated: Disallow functions that have Any in their signature after decorator transformation.
        disallow_any_explicit: Disallow explicit Any in type positions.
        disallow_any_generics: Disallow usage of generic types that do not specify explicit type parameters
            (inverse: --allow-any-generics).
        disallow_subclassing_any: Disallow subclassing values of type 'Any' when defining classes
            (inverse: --allow-subclassing-any).
        disallow_untyped_calls: Disallow calling functions without type annotations from functions with type annotations
            (inverse: --allow-untyped-calls).
        disallow_untyped_defs: Disallow defining functions without type annotations or with incomplete type annotations
            (inverse: --allow-untyped-defs).
        disallow_incomplete_defs: Disallow defining functions with incomplete type annotations
            (inverse: --allow-incomplete-defs).
        check_untyped_defs: Type check the interior of functions without type annotations
            (inverse: --no-check-untyped-defs).
        disallow_untyped_decorators: Disallow decorating typed functions with untyped decorators
            (inverse: --allow-untyped-decorators).
        implicit_optional: Assume arguments with default values of None are Optional(inverse: --no-implicit-optional).
        no_strict_optional: Disable strict Optional checks (inverse: --strict-optional).
        warn_redundant_casts: Warn about casting an expression to its inferred type (inverse: --no-warn-redundant-casts).
        warn_unused_ignores: Warn about unneeded '# type: ignore' comments (inverse: --no-warn-unused-ignores).
        no_warn_no_return: Do not warn about functions that end without returning (inverse: --warn-no-return).
        warn_return_any: Warn about returning values of type Any from non-Any typed functions (inverse: --no-warn-return-any).
        warn_unreachable: Warn about statements or expressions inferred to be unreachable (inverse: --no-warn-unreachable).
        allow_untyped_globals: Suppress toplevel errors caused by missing annotations (inverse: --disallow-untyped-globals).
        allow_redefinition: Allow unconditional variable redefinition with a new type (inverse: --disallow-redefinition).
        no_implicit_reexport: Treat imports as private unless aliased (inverse: --implicit-reexport).
        strict_equality: Prohibit equality, identity, and container checks for non-overlapping types
            (inverse: --no-strict-equality).
        strict_concatenate: Make arguments prepended via Concatenate be truly positional-only (inverse: --no-strict-concatenate).
        strict: Strict mode; enables the following flags: --warn-unused-configs, --disallow-any-generics,
            --disallow-subclassing-any, --disallow-untyped-calls, --disallow-untyped-defs, --disallow-incomplete-defs,
            --check-untyped-defs, --disallow-untyped-decorators, --warn-redundant-casts, --warn-unused-ignores,
            --warn-return-any, --no-implicit-reexport, --strict-equality, --strict-concatenate.
        disable_error_code: Disable a specific error code.
        enable_error_code: Enable a specific error code.
        show_error_context: Precede errors with "note:" messages explaining context (inverse: --hide-error-context).
        show_column_numbers: Show column numbers in error messages (inverse: --hide-column-numbers).
        show_error_end: Show end line/end column numbers in error messages. This implies --show-column-numbers
            (inverse: --hide-error-end).
        hide_error_codes: Hide error codes in error messages (inverse: --show-error-codes).
        pretty: Use visually nicer output in error messages: Use soft word wrap, show source code snippets,
            and show error location markers (inverse: --no-pretty).
        no_color_output: Do not colorize error messages (inverse: --color-output).
        no_error_summary: Do not show error stats summary (inverse: --error-summary).
        show_absolute_path: Show absolute paths to files (inverse: --hide-absolute-path).
        no_incremental: Disable module cache (inverse: --incremental).
        cache_dir: Store module cache info in the given folder in incremental mode (defaults to '.mypy_cache').
        sqlite_cache: Use a sqlite database to store the cache (inverse: --no-sqlite-cache).
        cache_fine_grained: Include fine-grained dependency information in the cache for the mypy daemon.
        skip_version_check: Allow using cache written by older mypy version.
        skip_cache_mtime_checks: Skip cache internal consistency checks based on mtime.
        pdb: Invoke pdb on fatal error.
        show_traceback: Show traceback on fatal error.
        raise_exceptions: Raise exception on fatal error.
        custom_typing_module: Use a custom typing module.
        disable_recursive_aliases: Disable experimental support for recursive type aliases.
        custom_typeshed_dir: Use the custom typeshed in DIR.
        warn_incomplete_stub: Warn if missing type annotation in typeshed, only relevant with --disallow-untyped-defs
            or --disallow-incomplete-defs enabled (inverse: --no-warn-incomplete-stub).
        shadow_file: When encountering SOURCE_FILE, read and type check the contents of SHADOW_FILE instead..
        any_exprs_report: Report any expression.
        cobertura_xml_report: Report Cobertura.
        html_report: Report HTML.
        linecount_report: Report line count.
        linecoverage_report: Report line coverage.
        lineprecision_report: Report line precision.
        txt_report: Report text.
        xml_report: Report XML.
        xslt_html_report: Report XLST HTML.
        xslt_txt_report: Report XLST text.
        junit_xml: Write junit.xml to the given file.
        find_occurrences: Print out all usages of a class member (experimental).
        scripts_are_modules: Script x becomes module x instead of __main__.
        install_types: Install detected missing library stub packages using pip (inverse: --no-install-types).
        non_interactive: Install stubs without asking for confirmation and hide errors, with --install-types
            (inverse: --interactive).
        explicit_package_bases: Use current directory and MYPYPATH to determine module names of files passed
            (inverse: --no-explicit-package-bases).
        exclude: Regular expression to match file names, directory names or paths which mypy should ignore while
            recursively discovering files to check, e.g. --exclude '/setup\\.py$'.
            May be specified more than once, eg. --exclude a --exclude b.
        module: Type-check module; can repeat for more modules.
        package: Type-check package recursively; can be repeated.
        command: Type-check program passed in as string.
    """  # noqa: D301
    from mypy.main import main as mypy  # noqa: PLC0415

    cli_args = list(paths)

    if enable_incomplete_feature:
        cli_args.append("--enable-incomplete-feature")

    if verbose:
        cli_args.append("--verbose")

    if config_file:
        cli_args.append("--config-file")
        cli_args.append(config_file)

    if warn_unused_configs:
        cli_args.append("--warn-unused-configs")

    if no_namespace_packages:
        cli_args.append("--no-namespace-packages")

    if ignore_missing_imports:
        cli_args.append("--ignore-missing-imports")

    if follow_imports:
        cli_args.append("--follow-imports")
        cli_args.append(follow_imports)

    if python_executable:
        cli_args.append("--python-executable")
        cli_args.append(python_executable)

    if no_site_packages:
        cli_args.append("--no-site-packages")

    if no_silence_site_packages:
        cli_args.append("--no-silence-site-packages")

    if python_version:
        cli_args.append("--python-version")
        cli_args.append(python_version)

    if py2:
        cli_args.append("--py2")

    if platform:
        cli_args.append("--platform")
        cli_args.append(platform)

    if always_true:
        for posarg in always_true:
            cli_args.append("--always-true")
            cli_args.append(posarg)

    if always_false:
        for posarg in always_false:
            cli_args.append("--always-false")
            cli_args.append(posarg)

    if disallow_any_unimported:
        cli_args.append("--disallow-any-unimported")

    if disallow_any_expr:
        cli_args.append("--disallow-any-expr")

    if disallow_any_decorated:
        cli_args.append("--disallow-any-decorated")

    if disallow_any_explicit:
        cli_args.append("--disallow-any-explicit")

    if disallow_any_generics:
        cli_args.append("--disallow-any-generics")

    if disallow_subclassing_any:
        cli_args.append("--disallow-subclassing-any")

    if disallow_untyped_calls:
        cli_args.append("--disallow-untyped-calls")

    if disallow_untyped_defs:
        cli_args.append("--disallow-untyped-defs")

    if disallow_incomplete_defs:
        cli_args.append("--disallow-incomplete-defs")

    if check_untyped_defs:
        cli_args.append("--check-untyped-defs")

    if disallow_untyped_decorators:
        cli_args.append("--disallow-untyped-decorators")

    if implicit_optional:
        cli_args.append("--implicit-optional")

    if no_strict_optional:
        cli_args.append("--no-strict-optional")

    if warn_redundant_casts:
        cli_args.append("--warn-redundant-casts")

    if warn_unused_ignores:
        cli_args.append("--warn-unused-ignores")

    if no_warn_no_return:
        cli_args.append("--no-warn-no-return")

    if warn_return_any:
        cli_args.append("--warn-return-any")

    if warn_unreachable:
        cli_args.append("--warn-unreachable")

    if allow_untyped_globals:
        cli_args.append("--allow-untyped-globals")

    if allow_redefinition:
        cli_args.append("--allow-redefinition")

    if no_implicit_reexport:
        cli_args.append("--no-implicit-reexport")

    if strict_equality:
        cli_args.append("--strict-equality")

    if strict_concatenate:
        cli_args.append("--strict-concatenate")

    if strict:
        cli_args.append("--strict")

    if disable_error_code:
        cli_args.append("--disable-error-code")
        cli_args.append(disable_error_code)

    if enable_error_code:
        cli_args.append("--enable-error-code")
        cli_args.append(enable_error_code)

    if show_error_context:
        cli_args.append("--show-error-context")

    if show_column_numbers:
        cli_args.append("--show-column-numbers")

    if show_error_end:
        cli_args.append("--show-error-end")

    if hide_error_codes:
        cli_args.append("--hide-error-codes")

    if pretty:
        cli_args.append("--pretty")

    if no_color_output:
        cli_args.append("--no-color-output")

    if no_error_summary:
        cli_args.append("--no-error-summary")

    if show_absolute_path:
        cli_args.append("--show-absolute-path")

    if no_incremental:
        cli_args.append("--no-incremental")

    if cache_dir:
        cli_args.append("--cache-dir")
        cli_args.append(cache_dir)

    if sqlite_cache:
        cli_args.append("--sqlite-cache")

    if cache_fine_grained:
        cli_args.append("--cache-fine-grained")

    if skip_version_check:
        cli_args.append("--skip-version-check")

    if skip_cache_mtime_checks:
        cli_args.append("--skip-cache-mtime-checks")

    if pdb:
        cli_args.append("--pdb")

    if show_traceback:
        cli_args.append("--show-traceback")

    if raise_exceptions:
        cli_args.append("--raise-exceptions")

    if custom_typing_module:
        cli_args.append("--custom-typing-module")
        cli_args.append(custom_typing_module)

    if disable_recursive_aliases:
        cli_args.append("--disable-recursive-aliases")

    if custom_typeshed_dir:
        cli_args.append("--custom-typeshed-dir")
        cli_args.append(custom_typeshed_dir)

    if warn_incomplete_stub:
        cli_args.append("--warn-incomplete-stub")

    if shadow_file:
        cli_args.append("--shadow-file")
        cli_args.extend(shadow_file)

    if any_exprs_report:
        cli_args.append("--any-exprs-report")
        cli_args.append(any_exprs_report)

    if cobertura_xml_report:
        cli_args.append("--cobertura-xml-report")
        cli_args.append(cobertura_xml_report)

    if html_report:
        cli_args.append("--html-report")
        cli_args.append(html_report)

    if linecount_report:
        cli_args.append("--linecount-report")
        cli_args.append(linecount_report)

    if linecoverage_report:
        cli_args.append("--linecoverage-report")
        cli_args.append(linecoverage_report)

    if lineprecision_report:
        cli_args.append("--lineprecision-report")
        cli_args.append(lineprecision_report)

    if txt_report:
        cli_args.append("--txt-report")
        cli_args.append(txt_report)

    if xml_report:
        cli_args.append("--xml-report")
        cli_args.append(xml_report)

    if xslt_html_report:
        cli_args.append("--xslt-html-report")
        cli_args.append(xslt_html_report)

    if xslt_txt_report:
        cli_args.append("--xslt-txt-report")
        cli_args.append(xslt_txt_report)

    if junit_xml:
        cli_args.append("--junit-xml")
        cli_args.append(junit_xml)

    if find_occurrences:
        cli_args.append("--find-occurrences")
        cli_args.append(find_occurrences)

    if scripts_are_modules:
        cli_args.append("--scripts-are-modules")

    if install_types:
        cli_args.append("--install-types")

    if non_interactive:
        cli_args.append("--non-interactive")

    if explicit_package_bases:
        cli_args.append("--explicit-package-bases")

    if exclude:
        cli_args.append("--exclude")
        cli_args.append(exclude)

    if module:
        cli_args.append("--module")
        cli_args.append(module)

    if package:
        cli_args.append("--package")
        cli_args.append(package)

    if command:
        cli_args.append("--command")
        cli_args.append(command)

    mypy(
        args=cli_args,
        stdout=_LazyStdout(),
        stderr=_LazyStderr(),
        clean_exit=True,
    )
